"""
Python module generated from Java source file org.bukkit.inventory.CookingRecipe

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Collections
from org.bukkit import Keyed
from org.bukkit import Material
from org.bukkit import NamespacedKey
from org.bukkit.inventory import *
from org.bukkit.inventory.recipe import CookingBookCategory
from typing import Any, Callable, Iterable, Tuple


class CookingRecipe(Recipe, Keyed):
    """
    Represents a cooking recipe.
    
    Type `<T>`: type of recipe
    """

    def __init__(self, key: "NamespacedKey", result: "ItemStack", source: "Material", experience: float, cookingTime: int):
        """
        Create a cooking recipe to craft the specified ItemStack.

        Arguments
        - key: The unique recipe key
        - result: The item you want the recipe to create.
        - source: The input material.
        - experience: The experience given by this recipe
        - cookingTime: The cooking time (in ticks)
        """
        ...


    def __init__(self, key: "NamespacedKey", result: "ItemStack", input: "RecipeChoice", experience: float, cookingTime: int):
        """
        Create a cooking recipe to craft the specified ItemStack.

        Arguments
        - key: The unique recipe key
        - result: The item you want the recipe to create.
        - input: The input choices.
        - experience: The experience given by this recipe
        - cookingTime: The cooking time (in ticks)
        """
        ...


    def setInput(self, input: "Material") -> "CookingRecipe":
        """
        Sets the input of this cooking recipe.

        Arguments
        - input: The input material.

        Returns
        - The changed recipe, so you can chain calls.
        """
        ...


    def getInput(self) -> "ItemStack":
        """
        Get the input material.

        Returns
        - The input material.
        """
        ...


    def setInputChoice(self, input: "RecipeChoice") -> "T":
        """
        Sets the input of this cooking recipe.

        Arguments
        - input: The input choice.

        Returns
        - The changed recipe, so you can chain calls.
        """
        ...


    def getInputChoice(self) -> "RecipeChoice":
        """
        Get the input choice.

        Returns
        - The input choice.
        """
        ...


    def getResult(self) -> "ItemStack":
        """
        Get the result of this recipe.

        Returns
        - The resulting stack.
        """
        ...


    def setExperience(self, experience: float) -> None:
        """
        Sets the experience given by this recipe.

        Arguments
        - experience: the experience level
        """
        ...


    def getExperience(self) -> float:
        """
        Get the experience given by this recipe.

        Returns
        - experience level
        """
        ...


    def setCookingTime(self, cookingTime: int) -> None:
        """
        Set the cooking time for this recipe in ticks.

        Arguments
        - cookingTime: new cooking time
        """
        ...


    def getCookingTime(self) -> int:
        """
        Get the cooking time for this recipe in ticks.

        Returns
        - cooking time
        """
        ...


    def getKey(self) -> "NamespacedKey":
        ...


    def getGroup(self) -> str:
        """
        Get the group of this recipe. Recipes with the same group may be grouped
        together when displayed in the client.

        Returns
        - recipe group. An empty string denotes no group. May not be null.
        """
        ...


    def setGroup(self, group: str) -> None:
        """
        Set the group of this recipe. Recipes with the same group may be grouped
        together when displayed in the client.

        Arguments
        - group: recipe group. An empty string denotes no group. May not be
        null.
        """
        ...


    def getCategory(self) -> "CookingBookCategory":
        """
        Gets the category which this recipe will appear in the recipe book under.
        
        Defaults to CookingBookCategory.MISC if not set.

        Returns
        - recipe book category
        """
        ...


    def setCategory(self, category: "CookingBookCategory") -> None:
        """
        Sets the category which this recipe will appear in the recipe book under.
        
        Defaults to CookingBookCategory.MISC if not set.

        Arguments
        - category: recipe book category
        """
        ...



"""
Python module generated from Java source file java.security.NoSuchAlgorithmException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.security import *
from typing import Any, Callable, Iterable, Tuple


class NoSuchAlgorithmException(GeneralSecurityException):

    def __init__(self):
        """
        Constructs a NoSuchAlgorithmException with no detail
        message. A detail message is a String that describes this
        particular exception.
        """
        ...


    def __init__(self, msg: str):
        """
        Constructs a NoSuchAlgorithmException with the specified
        detail message. A detail message is a String that describes
        this particular exception, which may, for example, specify which
        algorithm is not available.

        Arguments
        - msg: the detail message.
        """
        ...


    def __init__(self, message: str, cause: "Throwable"):
        """
        Creates a `NoSuchAlgorithmException` with the specified
        detail message and cause.

        Arguments
        - message: the detail message (which is saved for later retrieval
               by the .getMessage() method).
        - cause: the cause (which is saved for later retrieval by the
               .getCause() method).  (A `null` value is permitted,
               and indicates that the cause is nonexistent or unknown.)

        Since
        - 1.5
        """
        ...


    def __init__(self, cause: "Throwable"):
        """
        Creates a `NoSuchAlgorithmException` with the specified cause
        and a detail message of `(cause==null ? null : cause.toString())`
        (which typically contains the class and detail message of
        `cause`).

        Arguments
        - cause: the cause (which is saved for later retrieval by the
               .getCause() method).  (A `null` value is permitted,
               and indicates that the cause is nonexistent or unknown.)

        Since
        - 1.5
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerShowEntityEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerShowEntityEvent(PlayerEvent):
    """
    Called when a hidden entity is shown to a player.
    
    This event is only called when the entity's visibility status is actually
    changed.
    
    This event is called regardless of whether the entity was within tracking
    range.

    See
    - Player.showEntity(org.bukkit.plugin.Plugin, org.bukkit.entity.Entity)
    """

    def __init__(self, who: "Player", entity: "Entity"):
        ...


    def getEntity(self) -> "Entity":
        """
        Gets the entity which has been shown to the player.

        Returns
        - the shown entity
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.plugin.PluginDescriptionFile

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import ImmutableList
from com.google.common.collect import ImmutableMap
from com.google.common.collect import ImmutableSet
from java.io import InputStream
from java.io import Reader
from java.io import Writer
from java.util import Locale
from java.util.regex import Pattern
from org.bukkit.command import Command
from org.bukkit.command import CommandExecutor
from org.bukkit.command import CommandSender
from org.bukkit.command import PluginCommand
from org.bukkit.command import TabCompleter
from org.bukkit.permissions import Permissible
from org.bukkit.permissions import Permission
from org.bukkit.permissions import PermissionDefault
from org.bukkit.plugin import *
from org.bukkit.plugin.java import JavaPlugin
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml import Yaml
from org.yaml.snakeyaml.constructor import AbstractConstruct
from org.yaml.snakeyaml.constructor import SafeConstructor
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.representer import Representer
from typing import Any, Callable, Iterable, Tuple


class PluginDescriptionFile:
    """
    This type is the runtime-container for the information in the plugin.yml.
    All plugins must have a respective plugin.yml. For plugins written in java
    using the standard plugin loader, this file must be in the root of the jar
    file.
    
    When Bukkit loads a plugin, it needs to know some basic information about
    it. It reads this information from a YAML file, 'plugin.yml'. This file
    consists of a set of attributes, each defined on a new line and with no
    indentation.
    
    Every (almost* every) method corresponds with a specific entry in the
    plugin.yml. These are the **required** entries for every plugin.yml:
    
    - .getName() - `name`
    - .getVersion() - `version`
    - .getMain() - `main`
    
    
    Failing to include any of these items will throw an exception and cause the
    server to ignore your plugin.
    
    This is a list of the possible yaml keys, with specific details included in
    the respective method documentations:
    <table border=1>
    <caption>The description of the plugin.yml layout</caption>
    <tr>
        <th>Node</th>
        <th>Method</th>
        <th>Summary</th>
    </tr><tr>
        <td>`name`</td>
        <td>.getName()</td>
        <td>The unique name of plugin</td>
    </tr><tr>
        <td>`provides`</td>
        <td>.getProvides()</td>
        <td>The plugin APIs which this plugin provides</td>
    </tr><tr>
        <td>`version`</td>
        <td>.getVersion()</td>
        <td>A plugin revision identifier</td>
    </tr><tr>
        <td>`main`</td>
        <td>.getMain()</td>
        <td>The plugin's initial class file</td>
    </tr><tr>
        <td>`author``authors`</td>
        <td>.getAuthors()</td>
        <td>The plugin authors</td>
    </tr><tr>
        <td>`contributors`</td>
        <td>.getContributors()</td>
        <td>The plugin contributors</td>
    </tr><tr>
        <td>`description`</td>
        <td>.getDescription()</td>
        <td>Human readable plugin summary</td>
    </tr><tr>
        <td>`website`</td>
        <td>.getWebsite()</td>
        <td>The URL to the plugin's site</td>
    </tr><tr>
        <td>`prefix`</td>
        <td>.getPrefix()</td>
        <td>The token to prefix plugin log entries</td>
    </tr><tr>
        <td>`load`</td>
        <td>.getLoad()</td>
        <td>The phase of server-startup this plugin will load during</td>
    </tr><tr>
        <td>`depend`</td>
        <td>.getDepend()</td>
        <td>Other required plugins</td>
    </tr><tr>
        <td>`softdepend`</td>
        <td>.getSoftDepend()</td>
        <td>Other plugins that add functionality</td>
    </tr><tr>
        <td>`loadbefore`</td>
        <td>.getLoadBefore()</td>
        <td>The inverse softdepend</td>
    </tr><tr>
        <td>`commands`</td>
        <td>.getCommands()</td>
        <td>The commands the plugin will register</td>
    </tr><tr>
        <td>`permissions`</td>
        <td>.getPermissions()</td>
        <td>The permissions the plugin will register</td>
    </tr><tr>
        <td>`default-permission`</td>
        <td>.getPermissionDefault()</td>
        <td>The default Permission.getDefault() default permission
            state for defined .getPermissions() permissions the plugin
            will register</td>
    </tr><tr>
        <td>`awareness`</td>
        <td>.getAwareness()</td>
        <td>The concepts that the plugin acknowledges</td>
    </tr><tr>
        <td>`api-version`</td>
        <td>.getAPIVersion()</td>
        <td>The API version which this plugin was programmed against</td>
    </tr><tr>
        <td>`libraries`</td>
        <td>.getLibraries() ()</td>
        <td>The libraries to be linked with this plugin</td>
    </tr>
    </table>
    
    A plugin.yml example:<blockquote>```
    name: Inferno
    provides: [Hell]
    version: 1.4.1
    description: This plugin is so 31337. You can set yourself on fire.
    # We could place every author in the authors list, but chose not to for illustrative purposes
    # Also, having an author distinguishes that person as the project lead, and ensures their
    # name is displayed first
    author: CaptainInflamo
    authors: [Cogito, verrier, EvilSeph]
    contributors: [Choco, md_5]
    website: http://www.curse.com/server-mods/minecraft/myplugin
    
    main: com.captaininflamo.bukkit.inferno.Inferno
    depend: [NewFire, FlameWire]
    api-version: 1.13
    libraries:
        - com.squareup.okhttp3:okhttp:4.9.0
    
    commands:
     flagrate:
       description: Set yourself on fire.
       aliases: [combust_me, combustMe]
       permission: inferno.flagrate
       usage: Syntax error! Simply type /&lt;command&gt; to ignite yourself.
     burningdeaths:
       description: List how many times you have died by fire.
       aliases: [burning_deaths, burningDeaths]
       permission: inferno.burningdeaths
       usage: |
         /&lt;command&gt; [player]
         Example: /&lt;command&gt; - see how many times you have burned to death
         Example: /&lt;command&gt; CaptainIce - see how many times CaptainIce has burned to death
    
    permissions:
     inferno.*:
       description: Gives access to all Inferno commands
       children:
         inferno.flagrate: True
         inferno.burningdeaths: True
         inferno.burningdeaths.others: True
     inferno.flagrate:
       description: Allows you to ignite yourself
       default: True
     inferno.burningdeaths:
       description: Allows you to see how many times you have burned to death
       default: True
     inferno.burningdeaths.others:
       description: Allows you to see how many times others have burned to death
       default: op
       children:
         inferno.burningdeaths: True
    ```</blockquote>
    """

    def __init__(self, stream: "InputStream"):
        ...


    def __init__(self, reader: "Reader"):
        """
        Loads a PluginDescriptionFile from the specified reader

        Arguments
        - reader: The reader

        Raises
        - InvalidDescriptionException: If the PluginDescriptionFile is
            invalid
        """
        ...


    def __init__(self, pluginName: str, pluginVersion: str, mainClass: str):
        """
        Creates a new PluginDescriptionFile with the given detailed

        Arguments
        - pluginName: Name of this plugin
        - pluginVersion: Version of this plugin
        - mainClass: Full location of the main class of this plugin
        """
        ...


    def getName(self) -> str:
        """
        Gives the name of the plugin. This name is a unique identifier for
        plugins.
        
        - Must consist of all alphanumeric characters, underscores, hyphon,
            and period (a-z,A-Z,0-9, _.-). Any other character will cause the
            plugin.yml to fail loading.
        - Used to determine the name of the plugin's data folder. Data
            folders are placed in the ./plugins/ directory by default, but this
            behavior should not be relied on. Plugin.getDataFolder()
            should be used to reference the data folder.
        - It is good practice to name your jar the same as this, for example
            'MyPlugin.jar'.
        - Case sensitive.
        - The is the token referenced in .getDepend(), .getSoftDepend(), and .getLoadBefore().
        - Using spaces in the plugin's name is deprecated.
        
        
        In the plugin.yml, this entry is named `name`.
        
        Example:<blockquote>```name: MyPlugin```</blockquote>

        Returns
        - the name of the plugin
        """
        ...


    def getProvides(self) -> list[str]:
        """
        Gives the list of other plugin APIs which this plugin provides.
        These are usable for other plugins to depend on.
        
        - Must consist of all alphanumeric characters, underscores, hyphon,
            and period (a-z,A-Z,0-9, _.-). Any other character will cause the
            plugin.yml to fail loading.
        - A different plugin providing the same one or using it as their name
            will not result in the plugin to fail loading.
        - Case sensitive.
        - An entry of this list can be referenced in .getDepend(),
           .getSoftDepend(), and .getLoadBefore().
        - `provides` must be in <a
            href="https://en.wikipedia.org/wiki/YAML#Lists">YAML list
            format</a>.
        
        
        In the plugin.yml, this entry is named `provides`.
        
        Example:
        <blockquote>```provides:
        - OtherPluginName
        - OldPluginName```</blockquote>

        Returns
        - immutable list of the plugin APIs which this plugin provides
        """
        ...


    def getVersion(self) -> str:
        """
        Gives the version of the plugin.
        
        - Version is an arbitrary string, however the most common format is
            MajorRelease.MinorRelease.Build (eg: 1.4.1).
        - Typically you will increment this every time you release a new
            feature or bug fix.
        - Displayed when a user types `/version PluginName`
        
        
        In the plugin.yml, this entry is named `version`.
        
        Example:<blockquote>```version: 1.4.1```</blockquote>

        Returns
        - the version of the plugin
        """
        ...


    def getMain(self) -> str:
        """
        Gives the fully qualified name of the main class for a plugin. The
        format should follow the ClassLoader.loadClass(String) syntax
        to successfully be resolved at runtime. For most plugins, this is the
        class that extends JavaPlugin.
        
        - This must contain the full namespace including the class file
            itself.
        - If your namespace is `org.bukkit.plugin`, and your class
            file is called `MyPlugin` then this must be
            `org.bukkit.plugin.MyPlugin`
        - No plugin can use `org.bukkit.` as a base package for
            **any class**, including the main class.
        
        
        In the plugin.yml, this entry is named `main`.
        
        Example:
        <blockquote>```main: org.bukkit.plugin.MyPlugin```</blockquote>

        Returns
        - the fully qualified main class for the plugin
        """
        ...


    def getDescription(self) -> str:
        """
        Gives a human-friendly description of the functionality the plugin
        provides.
        
        - The description can have multiple lines.
        - Displayed when a user types `/version PluginName`
        
        
        In the plugin.yml, this entry is named `description`.
        
        Example:
        <blockquote>```description: This plugin is so 31337. You can set yourself on fire.```</blockquote>

        Returns
        - description of this plugin, or null if not specified
        """
        ...


    def getLoad(self) -> "PluginLoadOrder":
        """
        Gives the phase of server startup that the plugin should be loaded.
        
        - Possible values are in PluginLoadOrder.
        - Defaults to PluginLoadOrder.POSTWORLD.
        - Certain caveats apply to each phase.
        - When different, .getDepend(), .getSoftDepend(), and
            .getLoadBefore() become relative in order loaded per-phase.
            If a plugin loads at `STARTUP`, but a dependency loads
            at `POSTWORLD`, the dependency will not be loaded before
            the plugin is loaded.
        
        
        In the plugin.yml, this entry is named `load`.
        
        Example:<blockquote>```load: STARTUP```</blockquote>

        Returns
        - the phase when the plugin should be loaded
        """
        ...


    def getAuthors(self) -> list[str]:
        """
        Gives the list of authors for the plugin.
        
        - Gives credit to the developer.
        - Used in some server error messages to provide helpful feedback on
            who to contact when an error occurs.
        - A SpigotMC forum handle or email address is recommended.
        - Is displayed when a user types `/version PluginName`
        - `authors` must be in <a
            href="https://en.wikipedia.org/wiki/YAML#Lists">YAML list
            format</a>.
        
        
        In the plugin.yml, this has two entries, `author` and
        `authors`.
        
        Single author example:
        <blockquote>```author: CaptainInflamo```</blockquote>
        Multiple author example:
        <blockquote>```authors: [Cogito, verrier, EvilSeph]```</blockquote>
        When both are specified, author will be the first entry in the list, so
        this example:
        <blockquote>```author: Grum
        authors:
        - feildmaster
        - amaranth```</blockquote>
        Is equivilant to this example:
        ```authors: [Grum, feildmaster, aramanth]```

        Returns
        - an immutable list of the plugin's authors
        """
        ...


    def getContributors(self) -> list[str]:
        """
        Gives the list of contributors for the plugin.
        
        - Gives credit to those that have contributed to the plugin, though
            not enough so to warrant authorship.
        - Unlike .getAuthors(), contributors will not be mentioned in
        server error messages as a means of contact.
        - A SpigotMC forum handle or email address is recommended.
        - Is displayed when a user types `/version PluginName`
        - `contributors` must be in <a
            href="https://en.wikipedia.org/wiki/YAML#Lists">YAML list
            format</a>.
        
        
        Example:
        <blockquote>```authors: [Choco, md_5]```</blockquote>

        Returns
        - an immutable list of the plugin's contributors
        """
        ...


    def getWebsite(self) -> str:
        """
        Gives the plugin's or plugin's author's website.
        
        - A link to the Curse page that includes documentation and downloads
            is highly recommended.
        - Displayed when a user types `/version PluginName`
        
        
        In the plugin.yml, this entry is named `website`.
        
        Example:
        <blockquote>```website: http://www.curse.com/server-mods/minecraft/myplugin```</blockquote>

        Returns
        - description of this plugin, or null if not specified
        """
        ...


    def getDepend(self) -> list[str]:
        """
        Gives a list of other plugins that the plugin requires.
        
        - Use the value in the .getName() of the target plugin to
            specify the dependency.
        - If any plugin listed here is not found, your plugin will fail to
            load at startup.
        - If multiple plugins list each other in `depend`,
            creating a network with no individual plugin does not list another
            plugin in the <a
            href=https://en.wikipedia.org/wiki/Circular_dependency>network</a>,
            all plugins in that network will fail.
        - `depend` must be in <a
            href="https://en.wikipedia.org/wiki/YAML#Lists">YAML list
            format</a>.
        
        
        In the plugin.yml, this entry is named `depend`.
        
        Example:
        <blockquote>```depend:
        - OnePlugin
        - AnotherPlugin```</blockquote>

        Returns
        - immutable list of the plugin's dependencies
        """
        ...


    def getSoftDepend(self) -> list[str]:
        """
        Gives a list of other plugins that the plugin requires for full
        functionality. The PluginManager will make best effort to treat
        all entries here as if they were a .getDepend() dependency, but
        will never fail because of one of these entries.
        
        - Use the value in the .getName() of the target plugin to
            specify the dependency.
        - When an unresolvable plugin is listed, it will be ignored and does
            not affect load order.
        - When a circular dependency occurs (a network of plugins depending
            or soft-dependending each other), it will arbitrarily choose a
            plugin that can be resolved when ignoring soft-dependencies.
        - `softdepend` must be in <a
            href="https://en.wikipedia.org/wiki/YAML#Lists">YAML list
            format</a>.
        
        
        In the plugin.yml, this entry is named `softdepend`.
        
        Example:
        <blockquote>```softdepend: [OnePlugin, AnotherPlugin]```</blockquote>

        Returns
        - immutable list of the plugin's preferred dependencies
        """
        ...


    def getLoadBefore(self) -> list[str]:
        """
        Gets the list of plugins that should consider this plugin a
        soft-dependency.
        
        - Use the value in the .getName() of the target plugin to
            specify the dependency.
        - The plugin should load before any other plugins listed here.
        - Specifying another plugin here is strictly equivalent to having the
            specified plugin's .getSoftDepend() include .getName() this plugin.
        - `loadbefore` must be in <a
            href="https://en.wikipedia.org/wiki/YAML#Lists">YAML list
            format</a>.
        
        
        In the plugin.yml, this entry is named `loadbefore`.
        
        Example:
        <blockquote>```loadbefore:
        - OnePlugin
        - AnotherPlugin```</blockquote>

        Returns
        - immutable list of plugins that should consider this plugin a
            soft-dependency
        """
        ...


    def getPrefix(self) -> str:
        """
        Gives the token to prefix plugin-specific logging messages with.
        
        - This includes all messages using Plugin.getLogger().
        - If not specified, the server uses the plugin's .getName()
            name.
        - This should clearly indicate what plugin is being logged.
        
        
        In the plugin.yml, this entry is named `prefix`.
        
        Example:<blockquote>```prefix: ex-why-zee```</blockquote>

        Returns
        - the prefixed logging token, or null if not specified
        """
        ...


    def getCommands(self) -> dict[str, dict[str, "Object"]]:
        """
        Gives the map of command-name to command-properties. Each entry in this
        map corresponds to a single command and the respective values are the
        properties of the command. Each property, *with the exception of
        aliases*, can be defined at runtime using methods in PluginCommand and are defined here only as a convenience.
        <table border=1>
        <caption>The command section's description</caption>
        <tr>
            <th>Node</th>
            <th>Method</th>
            <th>Type</th>
            <th>Description</th>
            <th>Example</th>
        </tr><tr>
            <td>`description`</td>
            <td>PluginCommand.setDescription(String)</td>
            <td>String</td>
            <td>A user-friendly description for a command. It is useful for
                documentation purposes as well as in-game help.</td>
            <td><blockquote>```description: Set yourself on fire```</blockquote></td>
        </tr><tr>
            <td>`aliases`</td>
            <td>PluginCommand.setAliases(List)</td>
            <td>String or <a
                href="https://en.wikipedia.org/wiki/YAML#Lists">List</a> of
                strings</td>
            <td>Alternative command names, with special usefulness for commands
                that are already registered. *Aliases are not effective when
                defined at runtime,* so the plugin description file is the
                only way to have them properly defined.
                
                Note: Command aliases may not have a colon in them.</td>
            <td>Single alias format:
                <blockquote>```aliases: combust_me```</blockquote> or
                multiple alias format:
                <blockquote>```aliases: [combust_me, combustMe]```</blockquote></td>
        </tr><tr>
            <td>`permission`</td>
            <td>PluginCommand.setPermission(String)</td>
            <td>String</td>
            <td>The name of the Permission required to use the command.
                A user without the permission will receive the specified
                message (see PluginCommand.setPermissionMessage(String) below), or a
                standard one if no specific message is defined. Without the
                permission node, no PluginCommand.setExecutor(CommandExecutor) CommandExecutor or
                PluginCommand.setTabCompleter(TabCompleter) will be called.</td>
            <td><blockquote>```permission: inferno.flagrate```</blockquote></td>
        </tr><tr>
            <td>`permission-message`</td>
            <td>PluginCommand.setPermissionMessage(String)</td>
            <td>String</td>
            <td>
                - Displayed to a player that attempts to use a command, but
                    does not have the required permission. See PluginCommand.getPermission() above.
                - &lt;permission&gt; is a macro that is replaced with the
                    permission node required to use the command.
                - Using empty quotes is a valid way to indicate nothing
                    should be displayed to a player.
                </td>
            <td><blockquote>```permission-message: You do not have /&lt;permission&gt;```</blockquote></td>
        </tr><tr>
            <td>`usage`</td>
            <td>PluginCommand.setUsage(String)</td>
            <td>String</td>
            <td>This message is displayed to a player when the PluginCommand.setExecutor(CommandExecutor) CommandExecutor.onCommand(CommandSender, Command, String, String[]) returns False.
                &lt;command&gt; is a macro that is replaced the command issued.</td>
            <td><blockquote>```usage: Syntax error! Perhaps you meant /&lt;command&gt; PlayerName?```</blockquote>
                It is worth noting that to use a colon in a yaml, like
                ``usage: Usage: /god [player]'`, you need to
                <a href="http://yaml.org/spec/current.html#id2503232">surround
                the message with double-quote</a>:
                <blockquote>```usage: "Usage: /god [player]"```</blockquote></td>
        </tr>
        </table>
        The commands are structured as a hiearchy of <a
        href="http://yaml.org/spec/current.html#id2502325">nested mappings</a>.
        The primary (top-level, no intendentation) node is
        ``commands`', while each individual command name is
        indented, indicating it maps to some value (in our case, the
        properties of the table above).
        
        Here is an example bringing together the piecemeal examples above, as
        well as few more definitions:<blockquote>```
        commands:
         flagrate:
           description: Set yourself on fire.
           aliases: [combust_me, combustMe]
           permission: inferno.flagrate
           permission-message: You do not have /&lt;permission&gt;
           usage: Syntax error! Perhaps you meant /&lt;command&gt; PlayerName?
         burningdeaths:
           description: List how many times you have died by fire.
           aliases:
           - burning_deaths
           - burningDeaths
           permission: inferno.burningdeaths
           usage: |
             /&lt;command&gt; [player]
             Example: /&lt;command&gt; - see how many times you have burned to death
             Example: /&lt;command&gt; CaptainIce - see how many times CaptainIce has burned to death
         # The next command has no description, aliases, etc. defined, but is still valid
         # Having an empty declaration is useful for defining the description, permission, and messages from a configuration dynamically
         apocalypse:
        ```</blockquote>
        Note: Command names may not have a colon in their name.

        Returns
        - the commands this plugin will register
        """
        ...


    def getPermissions(self) -> list["Permission"]:
        """
        Gives the list of permissions the plugin will register at runtime,
        immediately proceding enabling. The format for defining permissions is
        a map from permission name to properties. To represent a map without
        any specific property, empty <a
        href="http://yaml.org/spec/current.html#id2502702">curly-braces</a> (
        `&#123;&#125;` ) may be used (as a null value is not
        accepted, unlike the .getCommands() commands above).
        
        A list of optional properties for permissions:
        <table border=1>
        <caption>The permission section's description</caption>
        <tr>
            <th>Node</th>
            <th>Description</th>
            <th>Example</th>
        </tr><tr>
            <td>`description`</td>
            <td>Plaintext (user-friendly) description of what the permission
                is for.</td>
            <td><blockquote>```description: Allows you to set yourself on fire```</blockquote></td>
        </tr><tr>
            <td>`default`</td>
            <td>The default state for the permission, as defined by Permission.getDefault(). If not defined, it will be set to
                the value of PluginDescriptionFile.getPermissionDefault().
                
                For reference:
                - `True` - Represents a positive assignment to
                    Permissible permissibles.
                - `False` - Represents no assignment to Permissible permissibles.
                - `op` - Represents a positive assignment to
                    Permissible.isOp() operator permissibles.
                - `notop` - Represents a positive assignment to
                    Permissible.isOp() non-operator permissibiles.
                </td>
            <td><blockquote>```default: True```</blockquote></td>
        </tr><tr>
            <td>`children`</td>
            <td>Allows other permissions to be set as a Permission.getChildren() relation to the parent permission.
                When a parent permissions is assigned, child permissions are
                respectively assigned as well.
                
                - When a parent permission is assigned negatively, child
                    permissions are assigned based on an inversion of their
                    association.
                - When a parent permission is assigned positively, child
                    permissions are assigned based on their association.
                
                
                Child permissions may be defined in a number of ways:
                - Children may be defined as a <a
                    href="https://en.wikipedia.org/wiki/YAML#Lists">list</a> of
                    names. Using a list will treat all children associated
                    positively to their parent.
                - Children may be defined as a map. Each permission name maps
                    to either a boolean (representing the association), or a
                    nested permission definition (just as another permission).
                    Using a nested definition treats the child as a positive
                    association.
                - A nested permission definition must be a map of these same
                    properties. To define a valid nested permission without
                    defining any specific property, empty curly-braces (
                    `&#123;&#125;` ) must be used.
                 - A nested permission may carry it's own nested permissions
                     as children, as they may also have nested permissions, and
                     so forth. There is no direct limit to how deep the
                     permission tree is defined.
                </td>
            <td>As a list:
                <blockquote>```children: [inferno.flagrate, inferno.burningdeaths]```</blockquote>
                Or as a mapping:
                <blockquote>```children:
         inferno.flagrate: True
         inferno.burningdeaths: True```</blockquote>
                An additional example showing basic nested values can be seen
                <a href="doc-files/permissions-example_plugin.yml">here</a>.
                </td>
        </tr>
        </table>
        The permissions are structured as a hiearchy of <a
        href="http://yaml.org/spec/current.html#id2502325">nested mappings</a>.
        The primary (top-level, no intendentation) node is
        ``permissions`', while each individual permission name is
        indented, indicating it maps to some value (in our case, the
        properties of the table above).
        
        Here is an example using some of the properties:<blockquote>```
        permissions:
         inferno.*:
           description: Gives access to all Inferno commands
           children:
             inferno.flagrate: True
             inferno.burningdeaths: True
         inferno.flagate:
           description: Allows you to ignite yourself
           default: True
         inferno.burningdeaths:
           description: Allows you to see how many times you have burned to death
           default: True
        ```</blockquote>
        Another example, with nested definitions, can be found <a
        href="doc-files/permissions-example_plugin.yml">here</a>.

        Returns
        - the permissions this plugin will register
        """
        ...


    def getPermissionDefault(self) -> "PermissionDefault":
        """
        Gives the default Permission.getDefault() default state of
        .getPermissions() permissions registered for the plugin.
        
        - If not specified, it will be PermissionDefault.OP.
        - It is matched using PermissionDefault.getByName(String)
        - It only affects permissions that do not define the
            `default` node.
        - It may be any value in PermissionDefault.
        
        
        In the plugin.yml, this entry is named `default-permission`.
        
        Example:<blockquote>```default-permission: NOT_OP```</blockquote>

        Returns
        - the default value for the plugin's permissions
        """
        ...


    def getAwareness(self) -> set["PluginAwareness"]:
        """
        Gives a set of every PluginAwareness for a plugin. An awareness
        dictates something that a plugin developer acknowledges when the plugin
        is compiled. Some implementions may define extra awarenesses that are
        not included in the API. Any unrecognized
        awareness (one unsupported or in a future version) will cause a dummy
        object to be created instead of failing.
        
        
        - Currently only supports the enumerated values in PluginAwareness.Flags.
        - Each awareness starts the identifier with bang-at
            (`!@`).
        - Unrecognized (future / unimplemented) entries are quietly replaced
            by a generic object that implements PluginAwareness.
        - A type of awareness must be defined by the runtime and acknowledged
            by the API, effectively discluding any derived type from any
            plugin's classpath.
        - `awareness` must be in <a
            href="https://en.wikipedia.org/wiki/YAML#Lists">YAML list
            format</a>.
        
        
        In the plugin.yml, this entry is named `awareness`.
        
        Example:<blockquote>```awareness:
        - !@UTF8```</blockquote>
        
        **Note:** Although unknown versions of some future awareness are
        gracefully substituted, previous versions of Bukkit (ones prior to the
        first implementation of awareness) will fail to load a plugin that
        defines any awareness.

        Returns
        - a set containing every awareness for the plugin
        """
        ...


    def getFullName(self) -> str:
        """
        Returns the name of a plugin, including the version. This method is
        provided for convenience; it uses the .getName() and .getVersion() entries.

        Returns
        - a descriptive name of the plugin and respective version
        """
        ...


    def getAPIVersion(self) -> str:
        """
        Gives the API version which this plugin is designed to support. No
        specific format is guaranteed.
        
        - Refer to release notes for supported API versions.
        
        
        In the plugin.yml, this entry is named `api-version`.
        
        Example:<blockquote>```api-version: 1.13```</blockquote>

        Returns
        - the version of the plugin
        """
        ...


    def getLibraries(self) -> list[str]:
        """
        Gets the libraries this plugin requires. This is a preview feature.
        
        - Libraries must be GAV specifiers and are loaded from Maven Central.
        
        
        Example:<blockquote>```libraries:
            - com.squareup.okhttp3:okhttp:4.9.0```</blockquote>

        Returns
        - required libraries
        """
        ...


    def getClassLoaderOf(self) -> str:
        """
        Returns
        - unused

        Deprecated
        - unused
        """
        ...


    def save(self, writer: "Writer") -> None:
        """
        Saves this PluginDescriptionFile to the given writer

        Arguments
        - writer: Writer to output this file to
        """
        ...


    def getRawName(self) -> str:
        """
        Returns
        - internal use

        Unknown Tags
        - Internal use
        """
        ...



"""
Python module generated from Java source file net.md_5.bungee.api.chat.TextComponent

Java source file obtained from artifact bungeecord-chat version 1.20-R0.2-20240119.213604-65

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Arrays
from java.util.function import Consumer
from java.util.regex import Matcher
from java.util.regex import Pattern
from net.md_5.bungee.api import ChatColor
from net.md_5.bungee.api.chat import *
from typing import Any, Callable, Iterable, Tuple


class TextComponent(BaseComponent):

    def __init__(self):
        """
        Creates a TextComponent with blank text.
        """
        ...


    def __init__(self, textComponent: "TextComponent"):
        """
        Creates a TextComponent with formatting and text from the passed
        component

        Arguments
        - textComponent: the component to copy from
        """
        ...


    def __init__(self, *extras: Tuple["BaseComponent", ...]):
        """
        Creates a TextComponent with blank text and the extras set to the passed
        array

        Arguments
        - extras: the extras to set
        """
        ...


    @staticmethod
    def fromLegacy(message: str) -> "BaseComponent":
        """
        Converts the old formatting system that used
        net.md_5.bungee.api.ChatColor.COLOR_CHAR into the new json based
        system.

        Arguments
        - message: the text to convert

        Returns
        - the components needed to print the message to the client
        """
        ...


    @staticmethod
    def fromLegacy(message: str, defaultColor: "ChatColor") -> "BaseComponent":
        """
        Converts the old formatting system that used
        net.md_5.bungee.api.ChatColor.COLOR_CHAR into the new json based
        system.

        Arguments
        - message: the text to convert
        - defaultColor: color to use when no formatting is to be applied
        (i.e. after ChatColor.RESET).

        Returns
        - the components needed to print the message to the client
        """
        ...


    @staticmethod
    def fromLegacyText(message: str) -> list["BaseComponent"]:
        """
        Converts the old formatting system that used
        net.md_5.bungee.api.ChatColor.COLOR_CHAR into the new json based
        system.

        Arguments
        - message: the text to convert

        Returns
        - the components needed to print the message to the client

        Deprecated
        - .fromLegacy(String) is preferred as it will
        consolidate all components into a single BaseComponent with extra
        contents as opposed to an array of components which is non-standard and
        may result in unexpected behavior.
        """
        ...


    @staticmethod
    def fromLegacyText(message: str, defaultColor: "ChatColor") -> list["BaseComponent"]:
        """
        Converts the old formatting system that used
        net.md_5.bungee.api.ChatColor.COLOR_CHAR into the new json based
        system.

        Arguments
        - message: the text to convert
        - defaultColor: color to use when no formatting is to be applied
        (i.e. after ChatColor.RESET).

        Returns
        - the components needed to print the message to the client

        Deprecated
        - .fromLegacy(String, ChatColor) is preferred as it
        will consolidate all components into a single BaseComponent with extra
        contents as opposed to an array of components which is non-standard and
        may result in unexpected behavior.
        """
        ...


    @staticmethod
    def fromArray(*components: Tuple["BaseComponent", ...]) -> "BaseComponent":
        """
        Internal compatibility method to transform an array of components to a
        single component.

        Arguments
        - components: array

        Returns
        - single component
        """
        ...


    def duplicate(self) -> "TextComponent":
        """
        Creates a duplicate of this TextComponent.

        Returns
        - the duplicate of this TextComponent.
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file java.util.concurrent.ThreadPoolExecutor

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import ConcurrentModificationException
from java.util import Iterator
from java.util.concurrent import *
from java.util.concurrent.atomic import AtomicInteger
from java.util.concurrent.locks import AbstractQueuedSynchronizer
from java.util.concurrent.locks import Condition
from java.util.concurrent.locks import ReentrantLock
from typing import Any, Callable, Iterable, Tuple


class ThreadPoolExecutor(AbstractExecutorService):
    """
    An ExecutorService that executes each submitted task using
    one of possibly several pooled threads, normally configured
    using Executors factory methods.
    
    Thread pools address two different problems: they usually
    provide improved performance when executing large numbers of
    asynchronous tasks, due to reduced per-task invocation overhead,
    and they provide a means of bounding and managing the resources,
    including threads, consumed when executing a collection of tasks.
    Each `ThreadPoolExecutor` also maintains some basic
    statistics, such as the number of completed tasks.
    
    To be useful across a wide range of contexts, this class
    provides many adjustable parameters and extensibility
    hooks. However, programmers are urged to use the more convenient
    Executors factory methods Executors.newCachedThreadPool (unbounded thread pool, with
    automatic thread reclamation), Executors.newFixedThreadPool
    (fixed size thread pool) and Executors.newSingleThreadExecutor (single background thread), that
    preconfigure settings for the most common usage
    scenarios. Otherwise, use the following guide when manually
    configuring and tuning this class:
    
    <dl>
    
    <dt>Core and maximum pool sizes</dt>
    
    <dd>A `ThreadPoolExecutor` will automatically adjust the
    pool size (see .getPoolSize)
    according to the bounds set by
    corePoolSize (see .getCorePoolSize) and
    maximumPoolSize (see .getMaximumPoolSize).
    
    When a new task is submitted in method .execute(Runnable),
    if fewer than corePoolSize threads are running, a new thread is
    created to handle the request, even if other worker threads are
    idle.  Else if fewer than maximumPoolSize threads are running, a
    new thread will be created to handle the request only if the queue
    is full.  By setting corePoolSize and maximumPoolSize the same, you
    create a fixed-size thread pool. By setting maximumPoolSize to an
    essentially unbounded value such as `Integer.MAX_VALUE`, you
    allow the pool to accommodate an arbitrary number of concurrent
    tasks. Most typically, core and maximum pool sizes are set only
    upon construction, but they may also be changed dynamically using
    .setCorePoolSize and .setMaximumPoolSize. </dd>
    
    <dt>On-demand construction</dt>
    
    <dd>By default, even core threads are initially created and
    started only when new tasks arrive, but this can be overridden
    dynamically using method .prestartCoreThread or .prestartAllCoreThreads.  You probably want to prestart threads if
    you construct the pool with a non-empty queue. </dd>
    
    <dt>Creating new threads</dt>
    
    <dd>New threads are created using a ThreadFactory.  If not
    otherwise specified, a Executors.defaultThreadFactory is
    used, that creates threads to all be in the same ThreadGroup and with the same `NORM_PRIORITY` priority and
    non-daemon status. By supplying a different ThreadFactory, you can
    alter the thread's name, thread group, priority, daemon status,
    etc. If a `ThreadFactory` fails to create a thread when asked
    by returning null from `newThread`, the executor will
    continue, but might not be able to execute any tasks. Threads
    should possess the "modifyThread" `RuntimePermission`. If
    worker threads or other threads using the pool do not possess this
    permission, service may be degraded: configuration changes may not
    take effect in a timely manner, and a shutdown pool may remain in a
    state in which termination is possible but not completed.</dd>
    
    <dt>Keep-alive times</dt>
    
    <dd>If the pool currently has more than corePoolSize threads,
    excess threads will be terminated if they have been idle for more
    than the keepAliveTime (see .getKeepAliveTime(TimeUnit)).
    This provides a means of reducing resource consumption when the
    pool is not being actively used. If the pool becomes more active
    later, new threads will be constructed. This parameter can also be
    changed dynamically using method .setKeepAliveTime(long,
    TimeUnit).  Using a value of `Long.MAX_VALUE` TimeUnit.NANOSECONDS effectively disables idle threads from ever
    terminating prior to shut down. By default, the keep-alive policy
    applies only when there are more than corePoolSize threads, but
    method .allowCoreThreadTimeOut(boolean) can be used to
    apply this time-out policy to core threads as well, so long as the
    keepAliveTime value is non-zero. </dd>
    
    <dt>Queuing</dt>
    
    <dd>Any BlockingQueue may be used to transfer and hold
    submitted tasks.  The use of this queue interacts with pool sizing:
    
    
    
    - If fewer than corePoolSize threads are running, the Executor
    always prefers adding a new thread
    rather than queuing.
    
    - If corePoolSize or more threads are running, the Executor
    always prefers queuing a request rather than adding a new
    thread.
    
    - If a request cannot be queued, a new thread is created unless
    this would exceed maximumPoolSize, in which case, the task will be
    rejected.
    
    
    
    There are three general strategies for queuing:
    <ol>
    
    - * Direct handoffs.* A good default choice for a work
    queue is a SynchronousQueue that hands off tasks to threads
    without otherwise holding them. Here, an attempt to queue a task
    will fail if no threads are immediately available to run it, so a
    new thread will be constructed. This policy avoids lockups when
    handling sets of requests that might have internal dependencies.
    Direct handoffs generally require unbounded maximumPoolSizes to
    avoid rejection of new submitted tasks. This in turn admits the
    possibility of unbounded thread growth when commands continue to
    arrive on average faster than they can be processed.
    
    - * Unbounded queues.* Using an unbounded queue (for
    example a LinkedBlockingQueue without a predefined
    capacity) will cause new tasks to wait in the queue when all
    corePoolSize threads are busy. Thus, no more than corePoolSize
    threads will ever be created. (And the value of the maximumPoolSize
    therefore doesn't have any effect.)  This may be appropriate when
    each task is completely independent of others, so tasks cannot
    affect each others execution; for example, in a web page server.
    While this style of queuing can be useful in smoothing out
    transient bursts of requests, it admits the possibility of
    unbounded work queue growth when commands continue to arrive on
    average faster than they can be processed.
    
    - *Bounded queues.* A bounded queue (for example, an
    ArrayBlockingQueue) helps prevent resource exhaustion when
    used with finite maximumPoolSizes, but can be more difficult to
    tune and control.  Queue sizes and maximum pool sizes may be traded
    off for each other: Using large queues and small pools minimizes
    CPU usage, OS resources, and context-switching overhead, but can
    lead to artificially low throughput.  If tasks frequently block (for
    example if they are I/O bound), a system may be able to schedule
    time for more threads than you otherwise allow. Use of small queues
    generally requires larger pool sizes, which keeps CPUs busier but
    may encounter unacceptable scheduling overhead, which also
    decreases throughput.
    
    </ol>
    
    </dd>
    
    <dt>Rejected tasks</dt>
    
    <dd>New tasks submitted in method .execute(Runnable) will be
    *rejected* when the Executor has been shut down, and also when
    the Executor uses finite bounds for both maximum threads and work queue
    capacity, and is saturated.  In either case, the `execute` method
    invokes the RejectedExecutionHandler.rejectedExecution(Runnable, ThreadPoolExecutor)
    method of its RejectedExecutionHandler.  Four predefined handler
    policies are provided:
    
    <ol>
    
    - In the default ThreadPoolExecutor.AbortPolicy, the handler
    throws a runtime RejectedExecutionException upon rejection.
    
    - In ThreadPoolExecutor.CallerRunsPolicy, the thread
    that invokes `execute` itself runs the task. This provides a
    simple feedback control mechanism that will slow down the rate that
    new tasks are submitted.
    
    - In ThreadPoolExecutor.DiscardPolicy, a task that cannot
    be executed is simply dropped. This policy is designed only for
    those rare cases in which task completion is never relied upon.
    
    - In ThreadPoolExecutor.DiscardOldestPolicy, if the
    executor is not shut down, the task at the head of the work queue
    is dropped, and then execution is retried (which can fail again,
    causing this to be repeated.) This policy is rarely acceptable.  In
    nearly all cases, you should also cancel the task to cause an
    exception in any component waiting for its completion, and/or log
    the failure, as illustrated in ThreadPoolExecutor.DiscardOldestPolicy documentation.
    
    </ol>
    
    It is possible to define and use other kinds of RejectedExecutionHandler classes. Doing so requires some care
    especially when policies are designed to work only under particular
    capacity or queuing policies. </dd>
    
    <dt>Hook methods</dt>
    
    <dd>This class provides `protected` overridable
    .beforeExecute(Thread, Runnable) and
    .afterExecute(Runnable, Throwable) methods that are called
    before and after execution of each task.  These can be used to
    manipulate the execution environment; for example, reinitializing
    ThreadLocals, gathering statistics, or adding log entries.
    Additionally, method .terminated can be overridden to perform
    any special processing that needs to be done once the Executor has
    fully terminated.
    
    If hook, callback, or BlockingQueue methods throw exceptions,
    internal worker threads may in turn fail, abruptly terminate, and
    possibly be replaced.</dd>
    
    <dt>Queue maintenance</dt>
    
    <dd>Method .getQueue() allows access to the work queue
    for purposes of monitoring and debugging.  Use of this method for
    any other purpose is strongly discouraged.  Two supplied methods,
    .remove(Runnable) and .purge are available to
    assist in storage reclamation when large numbers of queued tasks
    become cancelled.</dd>
    
    <dt>Reclamation</dt>
    
    <dd>A pool that is no longer referenced in a program *AND*
    has no remaining threads may be reclaimed (garbage collected)
    without being explicitly shutdown. You can configure a pool to
    allow all unused threads to eventually die by setting appropriate
    keep-alive times, using a lower bound of zero core threads and/or
    setting .allowCoreThreadTimeOut(boolean).  </dd>
    
    </dl>
    
    **Extension example.** Most extensions of this class
    override one or more of the protected hook methods. For example,
    here is a subclass that adds a simple pause/resume feature:
    
    ``` `class PausableThreadPoolExecutor extends ThreadPoolExecutor {
      private boolean isPaused;
      private ReentrantLock pauseLock = new ReentrantLock();
      private Condition unpaused = pauseLock.newCondition();
    
      public PausableThreadPoolExecutor(...) { super(...);`
    
      protected void beforeExecute(Thread t, Runnable r) {
        super.beforeExecute(t, r);
        pauseLock.lock();
        try {
          while (isPaused) unpaused.await();
        } catch (InterruptedException ie) {
          t.interrupt();
        } finally {
          pauseLock.unlock();
        }
      }
    
      public void pause() {
        pauseLock.lock();
        try {
          isPaused = True;
        } finally {
          pauseLock.unlock();
        }
      }
    
      public void resume() {
        pauseLock.lock();
        try {
          isPaused = False;
          unpaused.signalAll();
        } finally {
          pauseLock.unlock();
        }
      }
    }}```

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self, corePoolSize: int, maximumPoolSize: int, keepAliveTime: int, unit: "TimeUnit", workQueue: "BlockingQueue"["Runnable"]):
        """
        Creates a new `ThreadPoolExecutor` with the given initial
        parameters, the
        Executors.defaultThreadFactory default thread factory
        and the ThreadPoolExecutor.AbortPolicy
        default rejected execution handler.
        
        It may be more convenient to use one of the Executors
        factory methods instead of this general purpose constructor.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set
        - maximumPoolSize: the maximum number of threads to allow in the
               pool
        - keepAliveTime: when the number of threads is greater than
               the core, this is the maximum time that excess idle threads
               will wait for new tasks before terminating.
        - unit: the time unit for the `keepAliveTime` argument
        - workQueue: the queue to use for holding tasks before they are
               executed.  This queue will hold only the `Runnable`
               tasks submitted by the `execute` method.

        Raises
        - IllegalArgumentException: if one of the following holds:
                `corePoolSize < 0`
                `keepAliveTime < 0`
                `maximumPoolSize <= 0`
                `maximumPoolSize < corePoolSize`
        - NullPointerException: if `workQueue` is null
        """
        ...


    def __init__(self, corePoolSize: int, maximumPoolSize: int, keepAliveTime: int, unit: "TimeUnit", workQueue: "BlockingQueue"["Runnable"], threadFactory: "ThreadFactory"):
        """
        Creates a new `ThreadPoolExecutor` with the given initial
        parameters and the ThreadPoolExecutor.AbortPolicy
        default rejected execution handler.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set
        - maximumPoolSize: the maximum number of threads to allow in the
               pool
        - keepAliveTime: when the number of threads is greater than
               the core, this is the maximum time that excess idle threads
               will wait for new tasks before terminating.
        - unit: the time unit for the `keepAliveTime` argument
        - workQueue: the queue to use for holding tasks before they are
               executed.  This queue will hold only the `Runnable`
               tasks submitted by the `execute` method.
        - threadFactory: the factory to use when the executor
               creates a new thread

        Raises
        - IllegalArgumentException: if one of the following holds:
                `corePoolSize < 0`
                `keepAliveTime < 0`
                `maximumPoolSize <= 0`
                `maximumPoolSize < corePoolSize`
        - NullPointerException: if `workQueue`
                or `threadFactory` is null
        """
        ...


    def __init__(self, corePoolSize: int, maximumPoolSize: int, keepAliveTime: int, unit: "TimeUnit", workQueue: "BlockingQueue"["Runnable"], handler: "RejectedExecutionHandler"):
        """
        Creates a new `ThreadPoolExecutor` with the given initial
        parameters and the
        Executors.defaultThreadFactory default thread factory.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set
        - maximumPoolSize: the maximum number of threads to allow in the
               pool
        - keepAliveTime: when the number of threads is greater than
               the core, this is the maximum time that excess idle threads
               will wait for new tasks before terminating.
        - unit: the time unit for the `keepAliveTime` argument
        - workQueue: the queue to use for holding tasks before they are
               executed.  This queue will hold only the `Runnable`
               tasks submitted by the `execute` method.
        - handler: the handler to use when execution is blocked
               because the thread bounds and queue capacities are reached

        Raises
        - IllegalArgumentException: if one of the following holds:
                `corePoolSize < 0`
                `keepAliveTime < 0`
                `maximumPoolSize <= 0`
                `maximumPoolSize < corePoolSize`
        - NullPointerException: if `workQueue`
                or `handler` is null
        """
        ...


    def __init__(self, corePoolSize: int, maximumPoolSize: int, keepAliveTime: int, unit: "TimeUnit", workQueue: "BlockingQueue"["Runnable"], threadFactory: "ThreadFactory", handler: "RejectedExecutionHandler"):
        """
        Creates a new `ThreadPoolExecutor` with the given initial
        parameters.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set
        - maximumPoolSize: the maximum number of threads to allow in the
               pool
        - keepAliveTime: when the number of threads is greater than
               the core, this is the maximum time that excess idle threads
               will wait for new tasks before terminating.
        - unit: the time unit for the `keepAliveTime` argument
        - workQueue: the queue to use for holding tasks before they are
               executed.  This queue will hold only the `Runnable`
               tasks submitted by the `execute` method.
        - threadFactory: the factory to use when the executor
               creates a new thread
        - handler: the handler to use when execution is blocked
               because the thread bounds and queue capacities are reached

        Raises
        - IllegalArgumentException: if one of the following holds:
                `corePoolSize < 0`
                `keepAliveTime < 0`
                `maximumPoolSize <= 0`
                `maximumPoolSize < corePoolSize`
        - NullPointerException: if `workQueue`
                or `threadFactory` or `handler` is null
        """
        ...


    def execute(self, command: "Runnable") -> None:
        """
        Executes the given task sometime in the future.  The task
        may execute in a new thread or in an existing pooled thread.
        
        If the task cannot be submitted for execution, either because this
        executor has been shutdown or because its capacity has been reached,
        the task is handled by the current RejectedExecutionHandler.

        Arguments
        - command: the task to execute

        Raises
        - RejectedExecutionException: at discretion of
                `RejectedExecutionHandler`, if the task
                cannot be accepted for execution
        - NullPointerException: if `command` is null
        """
        ...


    def shutdown(self) -> None:
        """
        Initiates an orderly shutdown in which previously submitted
        tasks are executed, but no new tasks will be accepted.
        Invocation has no additional effect if already shut down.
        
        This method does not wait for previously submitted tasks to
        complete execution.  Use .awaitTermination awaitTermination
        to do that.

        Raises
        - SecurityException: 
        """
        ...


    def shutdownNow(self) -> list["Runnable"]:
        """
        Attempts to stop all actively executing tasks, halts the
        processing of waiting tasks, and returns a list of the tasks
        that were awaiting execution. These tasks are drained (removed)
        from the task queue upon return from this method.
        
        This method does not wait for actively executing tasks to
        terminate.  Use .awaitTermination awaitTermination to
        do that.
        
        There are no guarantees beyond best-effort attempts to stop
        processing actively executing tasks.  This implementation
        interrupts tasks via Thread.interrupt; any task that
        fails to respond to interrupts may never terminate.

        Raises
        - SecurityException: 
        """
        ...


    def isShutdown(self) -> bool:
        ...


    def isTerminating(self) -> bool:
        """
        Returns True if this executor is in the process of terminating
        after .shutdown or .shutdownNow but has not
        completely terminated.  This method may be useful for
        debugging. A return of `True` reported a sufficient
        period after shutdown may indicate that submitted tasks have
        ignored or suppressed interruption, causing this executor not
        to properly terminate.

        Returns
        - `True` if terminating but not yet terminated
        """
        ...


    def isTerminated(self) -> bool:
        ...


    def awaitTermination(self, timeout: int, unit: "TimeUnit") -> bool:
        ...


    def setThreadFactory(self, threadFactory: "ThreadFactory") -> None:
        """
        Sets the thread factory used to create new threads.

        Arguments
        - threadFactory: the new thread factory

        Raises
        - NullPointerException: if threadFactory is null

        See
        - .getThreadFactory
        """
        ...


    def getThreadFactory(self) -> "ThreadFactory":
        """
        Returns the thread factory used to create new threads.

        Returns
        - the current thread factory

        See
        - .setThreadFactory(ThreadFactory)
        """
        ...


    def setRejectedExecutionHandler(self, handler: "RejectedExecutionHandler") -> None:
        """
        Sets a new handler for unexecutable tasks.

        Arguments
        - handler: the new handler

        Raises
        - NullPointerException: if handler is null

        See
        - .getRejectedExecutionHandler
        """
        ...


    def getRejectedExecutionHandler(self) -> "RejectedExecutionHandler":
        """
        Returns the current handler for unexecutable tasks.

        Returns
        - the current handler

        See
        - .setRejectedExecutionHandler(RejectedExecutionHandler)
        """
        ...


    def setCorePoolSize(self, corePoolSize: int) -> None:
        """
        Sets the core number of threads.  This overrides any value set
        in the constructor.  If the new value is smaller than the
        current value, excess existing threads will be terminated when
        they next become idle.  If larger, new threads will, if needed,
        be started to execute any queued tasks.

        Arguments
        - corePoolSize: the new core size

        Raises
        - IllegalArgumentException: if `corePoolSize < 0`
                or `corePoolSize` is greater than the .getMaximumPoolSize() maximum pool size

        See
        - .getCorePoolSize
        """
        ...


    def getCorePoolSize(self) -> int:
        """
        Returns the core number of threads.

        Returns
        - the core number of threads

        See
        - .setCorePoolSize
        """
        ...


    def prestartCoreThread(self) -> bool:
        """
        Starts a core thread, causing it to idly wait for work. This
        overrides the default policy of starting core threads only when
        new tasks are executed. This method will return `False`
        if all core threads have already been started.

        Returns
        - `True` if a thread was started
        """
        ...


    def prestartAllCoreThreads(self) -> int:
        """
        Starts all core threads, causing them to idly wait for work. This
        overrides the default policy of starting core threads only when
        new tasks are executed.

        Returns
        - the number of threads started
        """
        ...


    def allowsCoreThreadTimeOut(self) -> bool:
        """
        Returns True if this pool allows core threads to time out and
        terminate if no tasks arrive within the keepAlive time, being
        replaced if needed when new tasks arrive. When True, the same
        keep-alive policy applying to non-core threads applies also to
        core threads. When False (the default), core threads are never
        terminated due to lack of incoming tasks.

        Returns
        - `True` if core threads are allowed to time out,
                else `False`

        Since
        - 1.6
        """
        ...


    def allowCoreThreadTimeOut(self, value: bool) -> None:
        """
        Sets the policy governing whether core threads may time out and
        terminate if no tasks arrive within the keep-alive time, being
        replaced if needed when new tasks arrive. When False, core
        threads are never terminated due to lack of incoming
        tasks. When True, the same keep-alive policy applying to
        non-core threads applies also to core threads. To avoid
        continual thread replacement, the keep-alive time must be
        greater than zero when setting `True`. This method
        should in general be called before the pool is actively used.

        Arguments
        - value: `True` if should time out, else `False`

        Raises
        - IllegalArgumentException: if value is `True`
                and the current keep-alive time is not greater than zero

        Since
        - 1.6
        """
        ...


    def setMaximumPoolSize(self, maximumPoolSize: int) -> None:
        """
        Sets the maximum allowed number of threads. This overrides any
        value set in the constructor. If the new value is smaller than
        the current value, excess existing threads will be
        terminated when they next become idle.

        Arguments
        - maximumPoolSize: the new maximum

        Raises
        - IllegalArgumentException: if the new maximum is
                less than or equal to zero, or
                less than the .getCorePoolSize core pool size

        See
        - .getMaximumPoolSize
        """
        ...


    def getMaximumPoolSize(self) -> int:
        """
        Returns the maximum allowed number of threads.

        Returns
        - the maximum allowed number of threads

        See
        - .setMaximumPoolSize
        """
        ...


    def setKeepAliveTime(self, time: int, unit: "TimeUnit") -> None:
        """
        Sets the thread keep-alive time, which is the amount of time
        that threads may remain idle before being terminated.
        Threads that wait this amount of time without processing a
        task will be terminated if there are more than the core
        number of threads currently in the pool, or if this pool
        .allowsCoreThreadTimeOut() allows core thread timeout.
        This overrides any value set in the constructor.

        Arguments
        - time: the time to wait.  A time value of zero will cause
               excess threads to terminate immediately after executing tasks.
        - unit: the time unit of the `time` argument

        Raises
        - IllegalArgumentException: if `time` less than zero or
                if `time` is zero and `allowsCoreThreadTimeOut`

        See
        - .getKeepAliveTime(TimeUnit)
        """
        ...


    def getKeepAliveTime(self, unit: "TimeUnit") -> int:
        """
        Returns the thread keep-alive time, which is the amount of time
        that threads may remain idle before being terminated.
        Threads that wait this amount of time without processing a
        task will be terminated if there are more than the core
        number of threads currently in the pool, or if this pool
        .allowsCoreThreadTimeOut() allows core thread timeout.

        Arguments
        - unit: the desired time unit of the result

        Returns
        - the time limit

        See
        - .setKeepAliveTime(long, TimeUnit)
        """
        ...


    def getQueue(self) -> "BlockingQueue"["Runnable"]:
        """
        Returns the task queue used by this executor. Access to the
        task queue is intended primarily for debugging and monitoring.
        This queue may be in active use.  Retrieving the task queue
        does not prevent queued tasks from executing.

        Returns
        - the task queue
        """
        ...


    def remove(self, task: "Runnable") -> bool:
        """
        Removes this task from the executor's internal queue if it is
        present, thus causing it not to be run if it has not already
        started.
        
        This method may be useful as one part of a cancellation
        scheme.  It may fail to remove tasks that have been converted
        into other forms before being placed on the internal queue.
        For example, a task entered using `submit` might be
        converted into a form that maintains `Future` status.
        However, in such cases, method .purge may be used to
        remove those Futures that have been cancelled.

        Arguments
        - task: the task to remove

        Returns
        - `True` if the task was removed
        """
        ...


    def purge(self) -> None:
        """
        Tries to remove from the work queue all Future
        tasks that have been cancelled. This method can be useful as a
        storage reclamation operation, that has no other impact on
        functionality. Cancelled tasks are never executed, but may
        accumulate in work queues until worker threads can actively
        remove them. Invoking this method instead tries to remove them now.
        However, this method may fail to remove tasks in
        the presence of interference by other threads.
        """
        ...


    def getPoolSize(self) -> int:
        """
        Returns the current number of threads in the pool.

        Returns
        - the number of threads
        """
        ...


    def getActiveCount(self) -> int:
        """
        Returns the approximate number of threads that are actively
        executing tasks.

        Returns
        - the number of threads
        """
        ...


    def getLargestPoolSize(self) -> int:
        """
        Returns the largest number of threads that have ever
        simultaneously been in the pool.

        Returns
        - the number of threads
        """
        ...


    def getTaskCount(self) -> int:
        """
        Returns the approximate total number of tasks that have ever been
        scheduled for execution. Because the states of tasks and
        threads may change dynamically during computation, the returned
        value is only an approximation.

        Returns
        - the number of tasks
        """
        ...


    def getCompletedTaskCount(self) -> int:
        """
        Returns the approximate total number of tasks that have
        completed execution. Because the states of tasks and threads
        may change dynamically during computation, the returned value
        is only an approximation, but one that does not ever decrease
        across successive calls.

        Returns
        - the number of tasks
        """
        ...


    def toString(self) -> str:
        """
        Returns a string identifying this pool, as well as its state,
        including indications of run state and estimated worker and
        task counts.

        Returns
        - a string identifying this pool, as well as its state
        """
        ...


    class CallerRunsPolicy(RejectedExecutionHandler):
        """
        A handler for rejected tasks that runs the rejected task
        directly in the calling thread of the `execute` method,
        unless the executor has been shut down, in which case the task
        is discarded.
        """

        def __init__(self):
            """
            Creates a `CallerRunsPolicy`.
            """
            ...


        def rejectedExecution(self, r: "Runnable", e: "ThreadPoolExecutor") -> None:
            """
            Executes task r in the caller's thread, unless the executor
            has been shut down, in which case the task is discarded.

            Arguments
            - r: the runnable task requested to be executed
            - e: the executor attempting to execute this task
            """
            ...


    class AbortPolicy(RejectedExecutionHandler):
        """
        A handler for rejected tasks that throws a
        RejectedExecutionException.
        
        This is the default handler for ThreadPoolExecutor and
        ScheduledThreadPoolExecutor.
        """

        def __init__(self):
            """
            Creates an `AbortPolicy`.
            """
            ...


        def rejectedExecution(self, r: "Runnable", e: "ThreadPoolExecutor") -> None:
            """
            Always throws RejectedExecutionException.

            Arguments
            - r: the runnable task requested to be executed
            - e: the executor attempting to execute this task

            Raises
            - RejectedExecutionException: always
            """
            ...


    class DiscardPolicy(RejectedExecutionHandler):
        """
        A handler for rejected tasks that silently discards the
        rejected task.
        """

        def __init__(self):
            """
            Creates a `DiscardPolicy`.
            """
            ...


        def rejectedExecution(self, r: "Runnable", e: "ThreadPoolExecutor") -> None:
            """
            Does nothing, which has the effect of discarding task r.

            Arguments
            - r: the runnable task requested to be executed
            - e: the executor attempting to execute this task
            """
            ...


    class DiscardOldestPolicy(RejectedExecutionHandler):
        """
        A handler for rejected tasks that discards the oldest unhandled
        request and then retries `execute`, unless the executor
        is shut down, in which case the task is discarded. This policy is
        rarely useful in cases where other threads may be waiting for
        tasks to terminate, or failures must be recorded. Instead consider
        using a handler of the form:
        ``` `new RejectedExecutionHandler() {
          public void rejectedExecution(Runnable r, ThreadPoolExecutor e) {
            Runnable dropped = e.getQueue().poll();
            if (dropped instanceof Future<?>) {
              ((Future<?>)dropped).cancel(False);
              // also consider logging the failure`
            e.execute(r);  // retry
        }}}```
        """

        def __init__(self):
            """
            Creates a `DiscardOldestPolicy` for the given executor.
            """
            ...


        def rejectedExecution(self, r: "Runnable", e: "ThreadPoolExecutor") -> None:
            """
            Obtains and ignores the next task that the executor
            would otherwise execute, if one is immediately available,
            and then retries execution of task r, unless the executor
            is shut down, in which case task r is instead discarded.

            Arguments
            - r: the runnable task requested to be executed
            - e: the executor attempting to execute this task
            """
            ...



"""
Python module generated from Java source file org.bukkit.CoalType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Maps
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class CoalType(Enum):
    """
    Represents the two types of coal
    """

    COAL = (0x0)
    CHARCOAL = (0x1)


    def getData(self) -> int:
        """
        Gets the associated data value representing this type of coal

        Returns
        - A byte containing the data value of this coal type

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getByData(data: int) -> "CoalType":
        """
        Gets the type of coal with the given data value

        Arguments
        - data: Data value to fetch

        Returns
        - The CoalType representing the given value, or null if
            it doesn't exist

        Deprecated
        - Magic value
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.BaseGraph

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.graph import *
from typing import Any, Callable, Iterable, Tuple


class BaseGraph(SuccessorsFunction, PredecessorsFunction):
    """
    A non-public interface for the methods shared between Graph and ValueGraph.
    
    Type `<N>`: Node parameter type

    Author(s)
    - James Sexton
    """

    def nodes(self) -> set["N"]:
        """
        Returns all nodes in this graph, in the order specified by .nodeOrder().
        """
        ...


    def edges(self) -> set["EndpointPair"["N"]]:
        """
        Returns all edges in this graph.
        """
        ...


    def isDirected(self) -> bool:
        """
        Returns True if the edges in this graph are directed. Directed edges connect a EndpointPair.source() source node to a EndpointPair.target() target node, while
        undirected edges connect a pair of nodes to each other.
        """
        ...


    def allowsSelfLoops(self) -> bool:
        """
        Returns True if this graph allows self-loops (edges that connect a node to itself). Attempting
        to add a self-loop to a graph that does not allow them will throw an IllegalArgumentException.
        """
        ...


    def nodeOrder(self) -> "ElementOrder"["N"]:
        """
        Returns the order of iteration for the elements of .nodes().
        """
        ...


    def incidentEdgeOrder(self) -> "ElementOrder"["N"]:
        """
        Returns an ElementOrder that specifies the order of iteration for the elements of
        .edges(), .adjacentNodes(Object), .predecessors(Object), .successors(Object) and .incidentEdges(Object).

        Since
        - 29.0
        """
        ...


    def adjacentNodes(self, node: "N") -> set["N"]:
        """
        Returns a live view of the nodes which have an incident edge in common with `node` in
        this graph.
        
        This is equal to the union of .predecessors(Object) and .successors(Object).
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def predecessors(self, node: "N") -> set["N"]:
        """
        Returns a live view of all nodes in this graph adjacent to `node` which can be reached by
        traversing `node`'s incoming edges *against* the direction (if any) of the edge.
        
        In an undirected graph, this is equivalent to .adjacentNodes(Object).
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def successors(self, node: "N") -> set["N"]:
        """
        Returns a live view of all nodes in this graph adjacent to `node` which can be reached by
        traversing `node`'s outgoing edges in the direction (if any) of the edge.
        
        In an undirected graph, this is equivalent to .adjacentNodes(Object).
        
        This is *not* the same as "all nodes reachable from `node` by following outgoing
        edges". For that functionality, see Graphs.reachableNodes(Graph, Object).
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def incidentEdges(self, node: "N") -> set["EndpointPair"["N"]]:
        """
        Returns a live view of the edges in this graph whose endpoints include `node`.
        
        This is equal to the union of incoming and outgoing edges.
        
        If `node` is removed from the graph after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the graph after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph

        Since
        - 24.0
        """
        ...


    def degree(self, node: "N") -> int:
        """
        Returns the count of `node`'s incident edges, counting self-loops twice (equivalently,
        the number of times an edge touches `node`).
        
        For directed graphs, this is equal to `inDegree(node) + outDegree(node)`.
        
        For undirected graphs, this is equal to `incidentEdges(node).size()` + (number of
        self-loops incident to `node`).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def inDegree(self, node: "N") -> int:
        """
        Returns the count of `node`'s incoming edges (equal to `predecessors(node).size()`)
        in a directed graph. In an undirected graph, returns the .degree(Object).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def outDegree(self, node: "N") -> int:
        """
        Returns the count of `node`'s outgoing edges (equal to `successors(node).size()`)
        in a directed graph. In an undirected graph, returns the .degree(Object).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this graph
        """
        ...


    def hasEdgeConnecting(self, nodeU: "N", nodeV: "N") -> bool:
        """
        Returns True if there is an edge that directly connects `nodeU` to `nodeV`. This is
        equivalent to `nodes().contains(nodeU) && successors(nodeU).contains(nodeV)`.
        
        In an undirected graph, this is equal to `hasEdgeConnecting(nodeV, nodeU)`.

        Since
        - 23.0
        """
        ...


    def hasEdgeConnecting(self, endpoints: "EndpointPair"["N"]) -> bool:
        """
        Returns True if there is an edge that directly connects `endpoints` (in the order, if
        any, specified by `endpoints`). This is equivalent to `edges().contains(endpoints)`.
        
        Unlike the other `EndpointPair`-accepting methods, this method does not throw if the
        endpoints are unordered; it simply returns False. This is for consistency with the behavior of
        Collection.contains(Object) (which does not generally throw if the object cannot be
        present in the collection), and the desire to have this method's behavior be compatible with
        `edges().contains(endpoints)`.

        Since
        - 27.1
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Crops

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import CropState
from org.bukkit import Material
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Crops(MaterialData):
    """
    Represents the different types of crops in different states of growth.

    See
    - Material.LEGACY_NETHER_WARTS

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        """
        Constructs a wheat crop block in the seeded state.
        """
        ...


    def __init__(self, state: "CropState"):
        """
        Constructs a wheat crop block in the given growth state

        Arguments
        - state: The growth state of the crops
        """
        ...


    def __init__(self, type: "Material", state: "CropState"):
        """
        Constructs a crop block of the given type and in the given growth state

        Arguments
        - type: The type of crops
        - state: The growth state of the crops
        """
        ...


    def __init__(self, type: "Material"):
        """
        Constructs a crop block of the given type and in the seeded state

        Arguments
        - type: The type of crops
        """
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def getState(self) -> "CropState":
        """
        Gets the current growth state of this crop
        
        For crops with only four growth states such as beetroot, only the values SEEDED, SMALL, TALL and RIPE will be
        returned.

        Returns
        - CropState of this crop
        """
        ...


    def setState(self, state: "CropState") -> None:
        """
        Sets the growth state of this crop
        
        For crops with only four growth states such as beetroot, the 8 CropStates are mapped into four states:
        
        SEEDED, SMALL, TALL and RIPE
        
        GERMINATED will change to SEEDED
        VERY_SMALL will change to SMALL
        MEDIUM will change to TALL
        VERY_TALL will change to RIPE

        Arguments
        - state: New growth state of this crop
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "Crops":
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockBreakEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockBreakEvent(BlockExpEvent, Cancellable):
    """
    Called when a block is broken by a player.
    
    If you wish to have the block drop experience, you must set the experience
    value above 0. By default, experience will be set in the event if:
    <ol>
    - The player is not in creative or adventure mode
    - The player can loot the block (ie: does not destroy it completely, by
        using the correct tool)
    - The player does not have silk touch
    - The block drops experience in vanilla Minecraft
    </ol>
    
    Note:
    Plugins wanting to simulate a traditional block drop should set the block
    to air and utilize their own methods for determining what the default drop
    for the block being broken is and what to do about it, if anything.
    
    If a Block Break event is cancelled, the block will not break and
    experience will not drop.
    """

    def __init__(self, theBlock: "Block", player: "Player"):
        ...


    def getPlayer(self) -> "Player":
        """
        Gets the Player that is breaking the block involved in this event.

        Returns
        - The Player that is breaking the block involved in this event
        """
        ...


    def setDropItems(self, dropItems: bool) -> None:
        """
        Sets whether or not the block will attempt to drop items as it normally
        would.
        
        If and only if this is False then BlockDropItemEvent will not be
        called after this event.

        Arguments
        - dropItems: Whether or not the block will attempt to drop items
        """
        ...


    def isDropItems(self) -> bool:
        """
        Gets whether or not the block will attempt to drop items.
        
        If and only if this is False then BlockDropItemEvent will not be
        called after this event.

        Returns
        - Whether or not the block will attempt to drop items
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.event.player.AsyncPlayerChatPreviewEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Warning
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class AsyncPlayerChatPreviewEvent(AsyncPlayerChatEvent):
    """
    Used to format chat for chat preview. If this event is used, then the result
    of the corresponding AsyncPlayerChatEvent **must** be formatted in
    the same way.

    Deprecated
    - chat previews have been removed
    """

    def __init__(self, async: bool, who: "Player", message: str, players: set["Player"]):
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.ItemMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Multimap
from org.bukkit import NamespacedKey
from org.bukkit import Tag
from org.bukkit.attribute import Attribute
from org.bukkit.attribute import AttributeModifier
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.damage import DamageType
from org.bukkit.enchantments import Enchantment
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemFactory
from org.bukkit.inventory import ItemFlag
from org.bukkit.inventory import ItemRarity
from org.bukkit.inventory import ItemStack
from org.bukkit.inventory.meta import *
from org.bukkit.inventory.meta.components import CustomModelDataComponent
from org.bukkit.inventory.meta.components import EquippableComponent
from org.bukkit.inventory.meta.components import FoodComponent
from org.bukkit.inventory.meta.components import JukeboxPlayableComponent
from org.bukkit.inventory.meta.components import ToolComponent
from org.bukkit.inventory.meta.components import UseCooldownComponent
from org.bukkit.inventory.meta.components.consumable import ConsumableComponent
from org.bukkit.inventory.meta.tags import CustomItemTagContainer
from org.bukkit.persistence import PersistentDataHolder
from org.bukkit.tag import DamageTypeTags
from typing import Any, Callable, Iterable, Tuple


class ItemMeta(Cloneable, ConfigurationSerializable, PersistentDataHolder):
    """
    This type represents the storage mechanism for auxiliary item data.
    
    An implementation will handle the creation and application for ItemMeta.
    This class should not be implemented by a plugin in a live environment.
    """

    def hasDisplayName(self) -> bool:
        """
        Checks for existence of a display name.

        Returns
        - True if this has a display name
        """
        ...


    def getDisplayName(self) -> str:
        """
        Gets the display name that is set.
        
        Plugins should check that hasDisplayName() returns `True`
        before calling this method.

        Returns
        - the display name that is set
        """
        ...


    def setDisplayName(self, name: str) -> None:
        """
        Sets the display name.

        Arguments
        - name: the name to set
        """
        ...


    def hasItemName(self) -> bool:
        """
        Checks for existence of an item name.
        
        Item name differs from display name in that it is cannot be edited by an
        anvil, is not styled with italics, and does not show labels.

        Returns
        - True if this has an item name
        """
        ...


    def getItemName(self) -> str:
        """
        Gets the item name that is set.
        
        Item name differs from display name in that it is cannot be edited by an
        anvil, is not styled with italics, and does not show labels.
        
        Plugins should check that hasItemName() returns `True` before
        calling this method.

        Returns
        - the item name that is set
        """
        ...


    def setItemName(self, name: str) -> None:
        """
        Sets the item name.
        
        Item name differs from display name in that it is cannot be edited by an
        anvil, is not styled with italics, and does not show labels.

        Arguments
        - name: the name to set
        """
        ...


    def hasLocalizedName(self) -> bool:
        """
        Checks for existence of a localized name.

        Returns
        - True if this has a localized name

        Deprecated
        - meta no longer exists
        """
        ...


    def getLocalizedName(self) -> str:
        """
        Gets the localized display name that is set.
        
        Plugins should check that hasLocalizedName() returns `True`
        before calling this method.

        Returns
        - the localized name that is set

        Deprecated
        - meta no longer exists
        """
        ...


    def setLocalizedName(self, name: str) -> None:
        """
        Sets the localized name.

        Arguments
        - name: the name to set

        Deprecated
        - meta no longer exists
        """
        ...


    def hasLore(self) -> bool:
        """
        Checks for existence of lore.

        Returns
        - True if this has lore
        """
        ...


    def getLore(self) -> list[str]:
        """
        Gets the lore that is set.
        
        Plugins should check if hasLore() returns `True` before
        calling this method.

        Returns
        - a list of lore that is set
        """
        ...


    def setLore(self, lore: list[str]) -> None:
        """
        Sets the lore for this item.
        Removes lore when given null.

        Arguments
        - lore: the lore that will be set
        """
        ...


    def hasCustomModelData(self) -> bool:
        """
        Checks for existence of custom model data.
        
        CustomModelData is an integer that may be associated client side with a
        custom item model.

        Returns
        - True if this has custom model data
        """
        ...


    def getCustomModelData(self) -> int:
        """
        Gets the custom model data that is set.
        
        CustomModelData is an integer that may be associated client side with a
        custom item model.
        
        Plugins should check that hasCustomModelData() returns `True`
        before calling this method.

        Returns
        - the custom model data that is set
        """
        ...


    def getCustomModelDataComponent(self) -> "CustomModelDataComponent":
        """
        Gets the custom model data set on this item, or creates an empty custom
        model data instance.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with
        .setCustomModelDataComponent(CustomModelDataComponent) to apply
        the changes.

        Returns
        - component
        """
        ...


    def setCustomModelData(self, data: "Integer") -> None:
        """
        Sets the custom model data.
        
        CustomModelData is an integer that may be associated client side with a
        custom item model.

        Arguments
        - data: the data to set, or null to clear
        """
        ...


    def setCustomModelDataComponent(self, customModelData: "CustomModelDataComponent") -> None:
        """
        Sets the custom model data component.

        Arguments
        - customModelData: new component
        """
        ...


    def hasEnchantable(self) -> bool:
        """
        Gets if the enchantable component is set.

        Returns
        - if an enchantable is set.
        """
        ...


    def getEnchantable(self) -> int:
        """
        Gets the enchantable component. Higher values allow higher enchantments.

        Returns
        - max_stack_size
        """
        ...


    def setEnchantable(self, enchantable: "Integer") -> None:
        """
        Sets the enchantable. Higher values allow higher enchantments.

        Arguments
        - enchantable: enchantable value
        """
        ...


    def hasEnchants(self) -> bool:
        """
        Checks for the existence of any enchantments.

        Returns
        - True if an enchantment exists on this meta
        """
        ...


    def hasEnchant(self, ench: "Enchantment") -> bool:
        """
        Checks for existence of the specified enchantment.

        Arguments
        - ench: enchantment to check

        Returns
        - True if this enchantment exists for this meta
        """
        ...


    def getEnchantLevel(self, ench: "Enchantment") -> int:
        """
        Checks for the level of the specified enchantment.

        Arguments
        - ench: enchantment to check

        Returns
        - The level that the specified enchantment has, or 0 if none
        """
        ...


    def getEnchants(self) -> dict["Enchantment", "Integer"]:
        """
        Returns a copy the enchantments in this ItemMeta. 
        Returns an empty map if none.

        Returns
        - An immutable copy of the enchantments
        """
        ...


    def addEnchant(self, ench: "Enchantment", level: int, ignoreLevelRestriction: bool) -> bool:
        """
        Adds the specified enchantment to this item meta.

        Arguments
        - ench: Enchantment to add
        - level: Level for the enchantment
        - ignoreLevelRestriction: this indicates the enchantment should be
            applied, ignoring the level limit

        Returns
        - True if the item meta changed as a result of this call, False
            otherwise
        """
        ...


    def removeEnchant(self, ench: "Enchantment") -> bool:
        """
        Removes the specified enchantment from this item meta.

        Arguments
        - ench: Enchantment to remove

        Returns
        - True if the item meta changed as a result of this call, False
            otherwise
        """
        ...


    def removeEnchantments(self) -> None:
        """
        Removes all enchantments from this item meta.
        """
        ...


    def hasConflictingEnchant(self, ench: "Enchantment") -> bool:
        """
        Checks if the specified enchantment conflicts with any enchantments in
        this ItemMeta.

        Arguments
        - ench: enchantment to test

        Returns
        - True if the enchantment conflicts, False otherwise
        """
        ...


    def addItemFlags(self, *itemFlags: Tuple["ItemFlag", ...]) -> None:
        """
        Set itemflags which should be ignored when rendering a ItemStack in the Client. This Method does silently ignore double set itemFlags.

        Arguments
        - itemFlags: The hideflags which shouldn't be rendered
        """
        ...


    def removeItemFlags(self, *itemFlags: Tuple["ItemFlag", ...]) -> None:
        """
        Remove specific set of itemFlags. This tells the Client it should render it again. This Method does silently ignore double removed itemFlags.

        Arguments
        - itemFlags: Hideflags which should be removed
        """
        ...


    def getItemFlags(self) -> set["ItemFlag"]:
        """
        Get current set itemFlags. The collection returned is unmodifiable.

        Returns
        - A set of all itemFlags set
        """
        ...


    def hasItemFlag(self, flag: "ItemFlag") -> bool:
        """
        Check if the specified flag is present on this item.

        Arguments
        - flag: the flag to check

        Returns
        - if it is present
        """
        ...


    def isHideTooltip(self) -> bool:
        """
        Gets if this item has hide_tooltip set. An item with this set will not
        show any tooltip whatsoever.

        Returns
        - hide_tooltip
        """
        ...


    def setHideTooltip(self, hideTooltip: bool) -> None:
        """
        Sets if this item has hide_tooltip set. An item with this set will not
        show any tooltip whatsoever.

        Arguments
        - hideTooltip: new hide_tooltip
        """
        ...


    def hasTooltipStyle(self) -> bool:
        """
        Gets if this item has a custom tooltip style.

        Returns
        - if a tooltip_style is set
        """
        ...


    def getTooltipStyle(self) -> "NamespacedKey":
        """
        Gets the custom tooltip style.

        Returns
        - the tooltip style
        """
        ...


    def setTooltipStyle(self, tooltipStyle: "NamespacedKey") -> None:
        """
        Sets the custom tooltip style.

        Arguments
        - tooltipStyle: the new style
        """
        ...


    def hasItemModel(self) -> bool:
        """
        Gets if this item has a custom item model.

        Returns
        - if a item_model is set
        """
        ...


    def getItemModel(self) -> "NamespacedKey":
        """
        Gets the custom item model.

        Returns
        - the item model
        """
        ...


    def setItemModel(self, itemModel: "NamespacedKey") -> None:
        """
        Sets the custom item model.

        Arguments
        - itemModel: the new model
        """
        ...


    def isUnbreakable(self) -> bool:
        """
        Return if the unbreakable tag is True. An unbreakable item will not lose
        durability.

        Returns
        - True if the unbreakable tag is True
        """
        ...


    def setUnbreakable(self, unbreakable: bool) -> None:
        """
        Sets the unbreakable tag. An unbreakable item will not lose durability.

        Arguments
        - unbreakable: True if set unbreakable
        """
        ...


    def hasEnchantmentGlintOverride(self) -> bool:
        """
        Gets if an enchantment_glint_override is set.

        Returns
        - if an enchantment_glint_override is set
        """
        ...


    def getEnchantmentGlintOverride(self) -> "Boolean":
        """
        Sets the enchantment_glint_override. If True, the item will glint, even
        without enchantments; if False, the item will not glint, even with
        enchantments.
        
        Plugins should check .hasEnchantmentGlintOverride() before
        calling this method.

        Returns
        - enchantment_glint_override
        """
        ...


    def setEnchantmentGlintOverride(self, override: "Boolean") -> None:
        """
        Sets the enchantment_glint_override. If True, the item will glint, even
        without enchantments; if False, the item will not glint, even with
        enchantments. If null, the override will be cleared.

        Arguments
        - override: new enchantment_glint_override
        """
        ...


    def isGlider(self) -> bool:
        """
        Checks if this item is a glider. If True, this item will allow players to
        glide when it is equipped.

        Returns
        - glider
        """
        ...


    def setGlider(self, glider: bool) -> None:
        """
        Sets if this item is a glider. If True, this item will allow players to
        glide when it is equipped.

        Arguments
        - glider: glider
        """
        ...


    def isFireResistant(self) -> bool:
        """
        Checks if this item is fire_resistant. If True, it will not burn in fire
        or lava.

        Returns
        - fire_resistant

        Deprecated
        - use .getDamageResistant() and
        DamageTypeTags.IS_FIRE
        """
        ...


    def setFireResistant(self, fireResistant: bool) -> None:
        """
        Sets if this item is fire_resistant. If True, it will not burn in fire
        or lava.

        Arguments
        - fireResistant: fire_resistant

        Deprecated
        - use .setDamageResistant(org.bukkit.Tag) and
        DamageTypeTags.IS_FIRE
        """
        ...


    def hasDamageResistant(self) -> bool:
        """
        Gets if this item is resistant to certain types of damage.

        Returns
        - True if a resistance is set
        """
        ...


    def getDamageResistant(self) -> "Tag"["DamageType"]:
        """
        Gets the type of damage this item will be resistant to when in entity
        form.
        
        Plugins should check .hasDamageResistant() before calling this
        method.

        Returns
        - damage type
        """
        ...


    def setDamageResistant(self, tag: "Tag"["DamageType"]) -> None:
        """
        Sets the type of damage this item will be resistant to when in entity
        form.

        Arguments
        - tag: the tag, or null to clear
        """
        ...


    def hasMaxStackSize(self) -> bool:
        """
        Gets if the max_stack_size is set.

        Returns
        - if a max_stack_size is set.
        """
        ...


    def getMaxStackSize(self) -> int:
        """
        Gets the max_stack_size. This is the maximum amount which an item will
        stack.

        Returns
        - max_stack_size
        """
        ...


    def setMaxStackSize(self, max: "Integer") -> None:
        """
        Sets the max_stack_size. This is the maximum amount which an item will
        stack.

        Arguments
        - max: max_stack_size, between 1 and 99 (inclusive)
        """
        ...


    def hasRarity(self) -> bool:
        """
        Gets if the rarity is set.

        Returns
        - rarity
        """
        ...


    def getRarity(self) -> "ItemRarity":
        """
        Gets the item rarity.
        
        Plugins should check .hasRarity() before calling this method.

        Returns
        - rarity
        """
        ...


    def setRarity(self, rarity: "ItemRarity") -> None:
        """
        Sets the item rarity.

        Arguments
        - rarity: new rarity
        """
        ...


    def hasUseRemainder(self) -> bool:
        """
        Checks if the use remainder is set.

        Returns
        - if a use remainder item is set
        """
        ...


    def getUseRemainder(self) -> "ItemStack":
        """
        Gets the item which this item will convert to when used.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with .setUseRemainder(ItemStack)
        to apply the changes.

        Returns
        - remainder
        """
        ...


    def setUseRemainder(self, remainder: "ItemStack") -> None:
        """
        Sets the item which this item will convert to when used.

        Arguments
        - remainder: new item
        """
        ...


    def hasUseCooldown(self) -> bool:
        """
        Checks if the use cooldown is set.

        Returns
        - if a use cooldown is set
        """
        ...


    def getUseCooldown(self) -> "UseCooldownComponent":
        """
        Gets the use cooldown set on this item, or creates an empty cooldown
        instance.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with
        .setUseCooldown(UseCooldownComponent) to apply the changes.

        Returns
        - cooldown
        """
        ...


    def setUseCooldown(self, cooldown: "UseCooldownComponent") -> None:
        """
        Sets the item use cooldown.

        Arguments
        - cooldown: new cooldown
        """
        ...


    def hasFood(self) -> bool:
        """
        Checks if the food is set.

        Returns
        - if a food is set
        """
        ...


    def getFood(self) -> "FoodComponent":
        """
        Gets the food set on this item, or creates an empty food instance.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with .setFood(FoodComponent) to
        apply the changes.

        Returns
        - food
        """
        ...


    def setFood(self, food: "FoodComponent") -> None:
        """
        Sets the item food.

        Arguments
        - food: new food
        """
        ...


    def hasConsumable(self) -> bool:
        """
        Checks if the consumable is set.

        Returns
        - if a consumable is set
        """
        ...


    def getConsumable(self) -> "ConsumableComponent":
        """
        Gets the consumable set on this item, or creates an empty consumable instance.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with .setConsumable(ConsumableComponent) to
        apply the changes.

        Returns
        - food
        """
        ...


    def setConsumable(self, consumable: "ConsumableComponent") -> None:
        """
        Sets the item consumable.

        Arguments
        - consumable: new consumable
        """
        ...


    def hasTool(self) -> bool:
        """
        Checks if the tool is set.

        Returns
        - if a tool is set
        """
        ...


    def getTool(self) -> "ToolComponent":
        """
        Gets the tool set on this item, or creates an empty tool instance.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with .setTool(ToolComponent) to
        apply the changes.

        Returns
        - tool
        """
        ...


    def setTool(self, tool: "ToolComponent") -> None:
        """
        Sets the item tool.

        Arguments
        - tool: new tool
        """
        ...


    def hasEquippable(self) -> bool:
        """
        Checks if the equippable is set.

        Returns
        - if a equippable is set
        """
        ...


    def getEquippable(self) -> "EquippableComponent":
        """
        Gets the equippable set on this item, or creates an empty equippable
        instance.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with
        .setEquippable(EquippableComponent) to apply the changes.

        Returns
        - equippable
        """
        ...


    def setEquippable(self, equippable: "EquippableComponent") -> None:
        """
        Sets the equippable tool.

        Arguments
        - equippable: new equippable
        """
        ...


    def hasJukeboxPlayable(self) -> bool:
        """
        Checks if the jukebox playable is set.

        Returns
        - if a jukebox playable is set
        """
        ...


    def getJukeboxPlayable(self) -> "JukeboxPlayableComponent":
        """
        Gets the jukebox playable component set on this item.
        
        The returned component is a snapshot of its current state and does not
        reflect a live view of what is on an item. After changing any value on
        this component, it must be set with
        .setJukeboxPlayable(org.bukkit.inventory.meta.components.JukeboxComponent)
        to apply the changes.

        Returns
        - component
        """
        ...


    def setJukeboxPlayable(self, jukeboxPlayable: "JukeboxPlayableComponent") -> None:
        """
        Sets the item tool.

        Arguments
        - jukeboxPlayable: new component
        """
        ...


    def hasAttributeModifiers(self) -> bool:
        """
        Checks for the existence of any AttributeModifiers.

        Returns
        - True if any AttributeModifiers exist
        """
        ...


    def getAttributeModifiers(self) -> "Multimap"["Attribute", "AttributeModifier"]:
        """
        Return an immutable copy of all Attributes and
        their modifiers in this ItemMeta.
        Returns null if none exist.

        Returns
        - an immutable Multimap of Attributes
                and their AttributeModifiers, or null if none exist
        """
        ...


    def getAttributeModifiers(self, slot: "EquipmentSlot") -> "Multimap"["Attribute", "AttributeModifier"]:
        """
        Return an immutable copy of all Attributes and their
        AttributeModifiers for a given EquipmentSlot.
        Any AttributeModifier that does have have a given
        EquipmentSlot will be returned. This is because
        AttributeModifiers without a slot are active in any slot.
        If there are no attributes set for the given slot, an empty map
        will be returned.

        Arguments
        - slot: the EquipmentSlot to check

        Returns
        - the immutable Multimap with the
                respective Attributes and modifiers, or an empty map
                if no attributes are set.
        """
        ...


    def getAttributeModifiers(self, attribute: "Attribute") -> Iterable["AttributeModifier"]:
        """
        Return an immutable copy of all AttributeModifiers
        for a given Attribute

        Arguments
        - attribute: the Attribute

        Returns
        - an immutable collection of AttributeModifiers
                 or null if no AttributeModifiers exist for the Attribute.

        Raises
        - NullPointerException: if Attribute is null
        """
        ...


    def addAttributeModifier(self, attribute: "Attribute", modifier: "AttributeModifier") -> bool:
        """
        Add an Attribute and it's Modifier.
        AttributeModifiers can now support EquipmentSlots.
        If not set, the AttributeModifier will be active in ALL slots.
        
        Two AttributeModifiers that have the same java.util.UUID
        cannot exist on the same Attribute.

        Arguments
        - attribute: the Attribute to modify
        - modifier: the AttributeModifier specifying the modification

        Returns
        - True if the Attribute and AttributeModifier were
                successfully added

        Raises
        - NullPointerException: if Attribute is null
        - NullPointerException: if AttributeModifier is null
        - IllegalArgumentException: if AttributeModifier already exists
        """
        ...


    def setAttributeModifiers(self, attributeModifiers: "Multimap"["Attribute", "AttributeModifier"]) -> None:
        """
        Set all Attributes and their AttributeModifiers.
        To clear all currently set Attributes and AttributeModifiers use
        null or an empty Multimap.
        If not null nor empty, this will filter all entries that are not-null
        and add them to the ItemStack.

        Arguments
        - attributeModifiers: the new Multimap containing the Attributes
                                  and their AttributeModifiers
        """
        ...


    def removeAttributeModifier(self, attribute: "Attribute") -> bool:
        """
        Remove all AttributeModifiers associated with the given
        Attribute.
        This will return False if nothing was removed.

        Arguments
        - attribute: attribute to remove

        Returns
        - True if all modifiers were removed from a given
                         Attribute. Returns False if no attributes were
                         removed.

        Raises
        - NullPointerException: if Attribute is null
        """
        ...


    def removeAttributeModifier(self, slot: "EquipmentSlot") -> bool:
        """
        Remove all Attributes and AttributeModifiers for a
        given EquipmentSlot.
        If the given EquipmentSlot is null, this will remove all
        AttributeModifiers that do not have an EquipmentSlot set.

        Arguments
        - slot: the EquipmentSlot to clear all Attributes and
                    their modifiers for

        Returns
        - True if all modifiers were removed that match the given
                EquipmentSlot.
        """
        ...


    def removeAttributeModifier(self, attribute: "Attribute", modifier: "AttributeModifier") -> bool:
        """
        Remove a specific Attribute and AttributeModifier.
        AttributeModifiers are matched according to their java.util.UUID.

        Arguments
        - attribute: the Attribute to remove
        - modifier: the AttributeModifier to remove

        Returns
        - if any attribute modifiers were remove

        Raises
        - NullPointerException: if the Attribute is null
        - NullPointerException: if the AttributeModifier is null

        See
        - AttributeModifier.getKey()
        """
        ...


    def getAsString(self) -> str:
        """
        Get this ItemMeta as an NBT string. If this ItemMeta does not have any
        NBT, then `"{`"} will be returned.
        
        This string should <strong>NEVER</strong> be relied upon as a serializable value. If
        serialization is desired, the ConfigurationSerializable API should be used
        instead.

        Returns
        - the NBT string
        """
        ...


    def getAsComponentString(self) -> str:
        """
        Get this ItemMeta as a component-compliant string. If this ItemMeta does
        not contain any components, then `"[]"` will be returned.
        
        The result of this method should yield a string representing the components
        altered by this ItemMeta instance. When passed to ItemFactory.createItemStack(String)
        with a prepended item type, it will create an ItemStack that has an ItemMeta
        matching this ItemMeta instance exactly. Note that this method returns <strong>
        ONLY</strong> the components and cannot be passed to createItemStack() alone.
        An example may look something like this:
        ```
        ItemStack itemStack = // ... an item stack obtained from somewhere
        ItemMeta itemMeta = itemStack.getItemMeta();
        
        String components = itemMeta.getAsComponentString(); // example: "[minecraft:damage=53]"
        String itemTypeKey = itemStack.getType().getKey().toString(); // example: "minecraft:diamond_sword"
        String itemAsString = itemTypeKey + components; // results in: "minecraft:diamond_sword[minecraft:damage=53]"
        
        ItemStack recreatedItemStack = Bukkit.getItemFactory().createItemStack(itemAsString);
        assert itemStack.isSimilar(recreatedItemStack); // Should be True*
        ```
        
        *Components not represented or explicitly overridden by this ItemMeta instance
        will not be included in the resulting string and therefore may result in ItemStacks
        that do not match *exactly*. For example, if .setDisplayName(String)
        is not set, then the custom name component will not be included. Or if this ItemMeta
        is a PotionMeta, it will not include any components related to lodestone compasses,
        banners, or books, etc., only components modifiable by a PotionMeta instance.
        
        This string should <strong>NEVER</strong> be relied upon as a serializable value. If
        serialization is desired, the ConfigurationSerializable API should be used
        instead.

        Returns
        - the component-compliant string
        """
        ...


    def getCustomTagContainer(self) -> "CustomItemTagContainer":
        """
        Returns a public custom tag container capable of storing tags on the
        item.
        
        Those tags will be sent to the client with all of their content, so the
        client is capable of reading them. This will result in the player seeing
        a NBT Tag notification on the item.
        
        These tags can also be modified by the client once in creative mode

        Returns
        - the custom tag container

        Deprecated
        - this API part has been replaced by the PersistentDataHolder API.
        Please use PersistentDataHolder.getPersistentDataContainer() instead of this.
        """
        ...


    def setVersion(self, version: int) -> None:
        """
        Internal use only! Do not use under any circumstances!

        Arguments
        - version: version

        Unknown Tags
        - internal use only
        """
        ...


    def clone(self) -> "ItemMeta":
        ...



"""
Python module generated from Java source file org.bukkit.configuration.MemoryConfiguration

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.configuration import *
from typing import Any, Callable, Iterable, Tuple


class MemoryConfiguration(MemorySection, Configuration):
    """
    This is a Configuration implementation that does not save or load
    from any source, and stores all values in memory only.
    This is useful for temporary Configurations for providing defaults.
    """

    def __init__(self):
        """
        Creates an empty MemoryConfiguration with no default values.
        """
        ...


    def __init__(self, defaults: "Configuration"):
        """
        Creates an empty MemoryConfiguration using the specified Configuration as a source for all default values.

        Arguments
        - defaults: Default value provider

        Raises
        - IllegalArgumentException: Thrown if defaults is null
        """
        ...


    def addDefault(self, path: str, value: "Object") -> None:
        ...


    def addDefaults(self, defaults: dict[str, "Object"]) -> None:
        ...


    def addDefaults(self, defaults: "Configuration") -> None:
        ...


    def setDefaults(self, defaults: "Configuration") -> None:
        ...


    def getDefaults(self) -> "Configuration":
        ...


    def getParent(self) -> "ConfigurationSection":
        ...


    def options(self) -> "MemoryConfigurationOptions":
        ...



"""
Python module generated from Java source file org.bukkit.event.vehicle.VehicleDamageEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import Vehicle
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.vehicle import *
from typing import Any, Callable, Iterable, Tuple


class VehicleDamageEvent(VehicleEvent, Cancellable):
    """
    Raised when a vehicle receives damage.
    """

    def __init__(self, vehicle: "Vehicle", attacker: "Entity", damage: float):
        ...


    def getAttacker(self) -> "Entity":
        """
        Gets the Entity that is attacking the vehicle

        Returns
        - the Entity that is attacking the vehicle
        """
        ...


    def getDamage(self) -> float:
        """
        Gets the damage done to the vehicle

        Returns
        - the damage done to the vehicle
        """
        ...


    def setDamage(self, damage: float) -> None:
        """
        Sets the damage done to the vehicle

        Arguments
        - damage: The damage
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.NullnessCasts

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.util.concurrent import *
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class NullnessCasts:
    """
    A utility method to perform unchecked casts to suppress errors produced by nullness analyses.
    """





"""
Python module generated from Java source file org.bukkit.block.data.type.BrewingStand

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import BlockData
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class BrewingStand(BlockData):
    """
    Interface to the 'has_bottle_0', 'has_bottle_1', 'has_bottle_2' flags on a
    brewing stand which indicate which bottles are rendered on the outside.
    
    Stand may have 0, 1... .getMaximumBottles()-1 bottles.
    """

    def hasBottle(self, bottle: int) -> bool:
        """
        Checks if the stand has the following bottle

        Arguments
        - bottle: to check

        Returns
        - if bottle is present
        """
        ...


    def setBottle(self, bottle: int, has: bool) -> None:
        """
        Set whether the stand has this bottle present.

        Arguments
        - bottle: to set
        - has: bottle
        """
        ...


    def getBottles(self) -> set["Integer"]:
        """
        Get the indexes of all the bottles present on this block.

        Returns
        - set of all bottles
        """
        ...


    def getMaximumBottles(self) -> int:
        """
        Get the maximum amount of bottles present on this stand.

        Returns
        - maximum bottle count
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.Directional

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import BlockFace
from org.bukkit.block.data import *
from typing import Any, Callable, Iterable, Tuple


class Directional(BlockData):
    """
    'facing' represents the face towards which the block is pointing.
    
    Some blocks may not be able to face in all directions, use
    .getFaces() to get all possible directions for this block.
    """

    def getFacing(self) -> "BlockFace":
        """
        Gets the value of the 'facing' property.

        Returns
        - the 'facing' value
        """
        ...


    def setFacing(self, facing: "BlockFace") -> None:
        """
        Sets the value of the 'facing' property.

        Arguments
        - facing: the new 'facing' value
        """
        ...


    def getFaces(self) -> set["BlockFace"]:
        """
        Gets the faces which are applicable to this block.

        Returns
        - the allowed 'facing' values
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Sniffer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import Location
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Sniffer(Animals):
    """
    Represents a Sniffer.
    """

    def getExploredLocations(self) -> Iterable["Location"]:
        """
        Gets the locations explored by the sniffer.
        
        **Note:** the returned locations use sniffer's current world.

        Returns
        - a collection of locations
        """
        ...


    def removeExploredLocation(self, location: "Location") -> None:
        """
        Remove a location of the explored locations.
        
        **Note:** the location must be in the sniffer's current world for this
        method to have any effect.

        Arguments
        - location: the location to remove

        See
        - .getExploredLocations()
        """
        ...


    def addExploredLocation(self, location: "Location") -> None:
        """
        Add a location to the explored locations.
        
        **Note:** the location must be in the sniffer's current world for this
        method to have any effect.

        Arguments
        - location: the location to add

        See
        - .getExploredLocations()
        """
        ...


    def getState(self) -> "Sniffer.State":
        """
        Get the current state of the sniffer.

        Returns
        - the state of the sniffer
        """
        ...


    def setState(self, state: "Sniffer.State") -> None:
        """
        Set a new state for the sniffer.
        
        This will also make the sniffer make the transition to the new state.

        Arguments
        - state: the new state
        """
        ...


    def findPossibleDigLocation(self) -> "Location":
        """
        Try to get a possible location where the sniffer can dig.

        Returns
        - a Location if found or null
        """
        ...


    def canDig(self) -> bool:
        """
        Gets whether the sniffer can dig in the current Location below
        its head.

        Returns
        - `True` if can dig or `False` otherwise
        """
        ...


    class State(Enum):
        """
        Represents the current state of the Sniffer.
        """

        IDLING = 0
        FEELING_HAPPY = 1
        SCENTING = 2
        SNIFFING = 3
        SEARCHING = 4
        DIGGING = 5
        RISING = 6



"""
Python module generated from Java source file com.google.common.reflect.Types

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Joiner
from com.google.common.base import Objects
from com.google.common.base import Predicates
from com.google.common.collect import ImmutableList
from com.google.common.collect import ImmutableMap
from com.google.common.collect import Iterables
from com.google.common.reflect import *
from java.io import Serializable
from java.lang.reflect import AnnotatedElement
from java.lang.reflect import Array
from java.lang.reflect import GenericArrayType
from java.lang.reflect import GenericDeclaration
from java.lang.reflect import InvocationHandler
from java.lang.reflect import InvocationTargetException
from java.lang.reflect import Method
from java.lang.reflect import ParameterizedType
from java.lang.reflect import Proxy
from java.lang.reflect import Type
from java.lang.reflect import TypeVariable
from java.lang.reflect import WildcardType
from java.security import AccessControlException
from java.util import Arrays
from java.util.concurrent.atomic import AtomicReference
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Types:
    """
    Utilities for working with Type.

    Author(s)
    - Ben Yu
    """





"""
Python module generated from Java source file org.yaml.snakeyaml.env.EnvScalarConstructor

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.regex import Matcher
from java.util.regex import Pattern
from org.yaml.snakeyaml import LoaderOptions
from org.yaml.snakeyaml import TypeDescription
from org.yaml.snakeyaml.constructor import AbstractConstruct
from org.yaml.snakeyaml.constructor import Constructor
from org.yaml.snakeyaml.env import *
from org.yaml.snakeyaml.error import MissingEnvironmentVariableException
from org.yaml.snakeyaml.nodes import Node
from org.yaml.snakeyaml.nodes import ScalarNode
from org.yaml.snakeyaml.nodes import Tag
from typing import Any, Callable, Iterable, Tuple


class EnvScalarConstructor(Constructor):
    """
    Construct scalar for format ${VARIABLE} replacing the template with the value from environment.
    It can also be used to create JavaBeans when the all the arguments are provided.

    See
    - <a href="https://docs.docker.com/compose/compose-file/.variable-substitution">Variable
         substitution</a>
    """

    ENV_TAG = Tag("!ENV")
    """
    Tag to indicate ENV
    """
    ENV_FORMAT = Pattern.compile("^\\$\\{\\s*((?<name>\\w+)((?<separator>:?(-|\\?))(?<value>\\S+)?)?)\\s*\\}$")
    """
    name must be a word ⇒ \w+ value can be any non-space ⇒ \S+
    """


    def __init__(self):
        """
        For simple cases when no JavaBeans are needed
        """
        ...


    def __init__(self, theRoot: "TypeDescription", moreTDs: Iterable["TypeDescription"], loadingConfig: "LoaderOptions"):
        """
        Create EnvScalarConstructor which can create JavaBeans with variable substitution

        Arguments
        - theRoot: - the class (usually JavaBean) to be constructed
        - moreTDs: - collection of classes used by the root class
        - loadingConfig: - configuration
        """
        ...


    def apply(self, name: str, separator: str, value: str, environment: str) -> str:
        """
        Implement the logic for missing and unset variables

        Arguments
        - name: - variable name in the template
        - separator: - separator in the template, can be :-, -, :?, ?
        - value: - default value or the error in the template
        - environment: - the value from environment for the provided variable

        Returns
        - the value to apply in the template
        """
        ...


    def getEnv(self, key: str) -> str:
        """
        Get value of the environment variable

        Arguments
        - key: - the name of the variable

        Returns
        - value or null if not set
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ListenerCallQueue

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Preconditions
from com.google.common.collect import Queues
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations.concurrent import GuardedBy
from java.util import Collections
from java.util import Queue
from java.util.concurrent import Executor
from typing import Any, Callable, Iterable, Tuple


class ListenerCallQueue:
    """
    A list of listeners for implementing a concurrency friendly observable object.
    
    Listeners are registered once via .addListener and then may be invoked by .enqueue enqueueing and then .dispatch dispatching events.
    
    The API of this class is designed to make it easy to achieve the following properties
    
    
      - Multiple events for the same listener are never dispatched concurrently.
      - Events for the different listeners are dispatched concurrently.
      - All events for a given listener dispatch on the provided .executor.
      - It is easy for the user to ensure that listeners are never invoked while holding locks.
    
    
    The last point is subtle. Often the observable object will be managing its own internal state
    using a lock, however it is dangerous to dispatch listeners while holding a lock because they
    might run on the `directExecutor()` or be otherwise re-entrant (call back into your
    object). So it is important to not call .dispatch while holding any locks. This is why
    .enqueue and .dispatch are 2 different methods. It is expected that the decision
    to run a particular event is made during the state change, but the decision to actually invoke
    the listeners can be delayed slightly so that locks can be dropped. Also, because .dispatch is expected to be called concurrently, it is idempotent.
    """

    def addListener(self, listener: "L", executor: "Executor") -> None:
        """
        Adds a listener that will be called using the given executor when events are later .enqueue enqueued and .dispatch dispatched.
        """
        ...


    def enqueue(self, event: "Event"["L"]) -> None:
        """
        Enqueues an event to be run on currently known listeners.
        
        The `toString` method of the Event itself will be used to describe the event in the
        case of an error.

        Arguments
        - event: the callback to execute on .dispatch
        """
        ...


    def enqueue(self, event: "Event"["L"], label: str) -> None:
        """
        Enqueues an event to be run on currently known listeners, with a label.

        Arguments
        - event: the callback to execute on .dispatch
        - label: a description of the event to use in the case of an error
        """
        ...


    def dispatch(self) -> None:
        """
        Dispatches all events enqueued prior to this call, serially and in order, for every listener.
        
        Note: this method is idempotent and safe to call from any thread
        """
        ...



"""
Python module generated from Java source file org.joml.Options

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.text import DecimalFormat
from java.text import NumberFormat
from java.util import Arrays
from java.util import Locale
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Options:
    """
    Utility class for reading system properties.

    Author(s)
    - Kai Burjack
    """

    DEBUG = hasOption(System.getProperty("joml.debug", "false"))
    """
    Whether certain debugging checks should be made, such as that only direct NIO Buffers are used when Unsafe is active,
    and a proxy should be created on calls to readOnlyView().
    """
    NO_UNSAFE = hasOption(System.getProperty("joml.nounsafe", "false"))
    """
    Whether *not* to use sun.misc.Unsafe when copying memory with MemUtil.
    """
    FORCE_UNSAFE = hasOption(System.getProperty("joml.forceUnsafe", "false"))
    """
    Whether to *force* the use of sun.misc.Unsafe when copying memory with MemUtil.
    """
    FASTMATH = hasOption(System.getProperty("joml.fastmath", "false"))
    """
    Whether fast approximations of some java.lang.Math operations should be used.
    """
    SIN_LOOKUP = hasOption(System.getProperty("joml.sinLookup", "false"))
    """
    When .FASTMATH is `True`, whether to use a lookup table for sin/cos.
    """
    SIN_LOOKUP_BITS = Integer.parseInt(System.getProperty("joml.sinLookup.bits", "14"))
    """
    When .SIN_LOOKUP is `True`, this determines the table size.
    """
    useNumberFormat = hasOption(System.getProperty("joml.format", "true"))
    """
    Whether to use a NumberFormat producing scientific notation output when formatting matrix,
    vector and quaternion components to strings.
    """
    USE_MATH_FMA = hasOption(System.getProperty("joml.useMathFma", "false"))
    """
    Whether to try using java.lang.Math.fma() in most matrix/vector/quaternion operations if it is available.
    If the CPU does *not* support it, it will be a lot slower than `a*b+c` and potentially generate a lot of memory allocations
    for the emulation with `java.util.BigDecimal`, though.
    """
    numberFormatDecimals = Integer.parseInt(System.getProperty("joml.format.decimals", "3"))
    """
    When .useNumberFormat is `True` then this determines the number of decimal digits
    produced in the formatted numbers.
    """
    NUMBER_FORMAT = decimalFormat()
    """
    The NumberFormat used to format all numbers throughout all JOML classes.
    """



"""
Python module generated from Java source file org.bukkit.entity.Panda

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Panda(Animals, Sittable):
    """
    Panda entity.
    """

    def getMainGene(self) -> "Gene":
        """
        Gets this Panda's main gene.

        Returns
        - main gene
        """
        ...


    def setMainGene(self, gene: "Gene") -> None:
        """
        Sets this Panda's main gene.

        Arguments
        - gene: main gene
        """
        ...


    def getHiddenGene(self) -> "Gene":
        """
        Gets this Panda's hidden gene.

        Returns
        - hidden gene
        """
        ...


    def setHiddenGene(self, gene: "Gene") -> None:
        """
        Sets this Panda's hidden gene.

        Arguments
        - gene: hidden gene
        """
        ...


    def isRolling(self) -> bool:
        """
        Gets whether the Panda is rolling

        Returns
        - Whether the Panda is rolling
        """
        ...


    def setRolling(self, flag: bool) -> None:
        """
        Sets whether the Panda is rolling

        Arguments
        - flag: Whether the Panda is rolling
        """
        ...


    def isSneezing(self) -> bool:
        """
        Gets whether the Panda is sneezing

        Returns
        - Whether the Panda is sneezing
        """
        ...


    def setSneezing(self, flag: bool) -> None:
        """
        Sets whether the Panda is sneezing

        Arguments
        - flag: Whether the Panda is sneezing
        """
        ...


    def isOnBack(self) -> bool:
        """
        Gets whether the Panda is on its back

        Returns
        - Whether the Panda is on its back
        """
        ...


    def setOnBack(self, flag: bool) -> None:
        """
        Sets whether the Panda is on its back

        Arguments
        - flag: Whether the Panda is on its back
        """
        ...


    def isEating(self) -> bool:
        """
        Gets whether the Panda is eating

        Returns
        - Whether the Panda is eating
        """
        ...


    def setEating(self, flag: bool) -> None:
        """
        Sets the Panda's eating status. The panda must be holding food for this to work

        Arguments
        - flag: Whether the Panda is eating
        """
        ...


    def isScared(self) -> bool:
        """
        Gets whether the Panda is scared

        Returns
        - Whether the Panda is scared
        """
        ...


    def getUnhappyTicks(self) -> int:
        """
        Gets how many ticks the panda will be unhappy for

        Returns
        - The number of ticks the panda will be unhappy for
        """
        ...


    class Gene(Enum):

        NORMAL = (False)
        LAZY = (False)
        WORRIED = (False)
        PLAYFUL = (False)
        BROWN = (True)
        WEAK = (True)
        AGGRESSIVE = (False)


        def isRecessive(self) -> bool:
            """
            Gets whether this gene is recessive, i.e. required in both parents to
            propagate to children.

            Returns
            - recessive status
            """
            ...



"""
Python module generated from Java source file com.google.common.io.ByteStreams

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from com.google.common.math import IntMath
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import ByteArrayInputStream
from java.io import ByteArrayOutputStream
from java.io import DataInput
from java.io import DataInputStream
from java.io import DataOutput
from java.io import DataOutputStream
from java.io import EOFException
from java.io import FilterInputStream
from java.io import IOException
from java.io import InputStream
from java.io import OutputStream
from java.util import ArrayDeque
from java.util import Arrays
from java.util import Queue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ByteStreams:
    """
    Provides utility methods for working with byte arrays and I/O streams.

    Author(s)
    - Colin Decker

    Since
    - 1.0
    """

    @staticmethod
    def copy(from: "InputStream", to: "OutputStream") -> int:
        """
        Copies all bytes from the input stream to the output stream. Does not close or flush either
        stream.
        
        **Java 9 users and later:** this method should be treated as deprecated; use the
        equivalent InputStream.transferTo method instead.

        Arguments
        - from: the input stream to read from
        - to: the output stream to write to

        Returns
        - the number of bytes copied

        Raises
        - IOException: if an I/O error occurs
        """
        ...


    @staticmethod
    def copy(from: "ReadableByteChannel", to: "WritableByteChannel") -> int:
        """
        Copies all bytes from the readable channel to the writable channel. Does not close or flush
        either channel.

        Arguments
        - from: the readable channel to read from
        - to: the writable channel to write to

        Returns
        - the number of bytes copied

        Raises
        - IOException: if an I/O error occurs
        """
        ...


    @staticmethod
    def toByteArray(in: "InputStream") -> list[int]:
        """
        Reads all bytes from an input stream into a byte array. Does not close the stream.
        
        **Java 9+ users:** use `in.readAllBytes()` instead.

        Arguments
        - in: the input stream to read from

        Returns
        - a byte array containing all the bytes from the stream

        Raises
        - IOException: if an I/O error occurs
        """
        ...


    @staticmethod
    def exhaust(in: "InputStream") -> int:
        """
        Reads and discards data from the given `InputStream` until the end of the stream is
        reached. Returns the total number of bytes read. Does not close the stream.

        Since
        - 20.0
        """
        ...


    @staticmethod
    def newDataInput(bytes: list[int]) -> "ByteArrayDataInput":
        """
        Returns a new ByteArrayDataInput instance to read from the `bytes` array from the
        beginning.
        """
        ...


    @staticmethod
    def newDataInput(bytes: list[int], start: int) -> "ByteArrayDataInput":
        """
        Returns a new ByteArrayDataInput instance to read from the `bytes` array,
        starting at the given position.

        Raises
        - IndexOutOfBoundsException: if `start` is negative or greater than the length of
            the array
        """
        ...


    @staticmethod
    def newDataInput(byteArrayInputStream: "ByteArrayInputStream") -> "ByteArrayDataInput":
        """
        Returns a new ByteArrayDataInput instance to read from the given `ByteArrayInputStream`. The given input stream is not reset before being read from by the
        returned `ByteArrayDataInput`.

        Since
        - 17.0
        """
        ...


    @staticmethod
    def newDataOutput() -> "ByteArrayDataOutput":
        """
        Returns a new ByteArrayDataOutput instance with a default size.
        """
        ...


    @staticmethod
    def newDataOutput(size: int) -> "ByteArrayDataOutput":
        """
        Returns a new ByteArrayDataOutput instance sized to hold `size` bytes before
        resizing.

        Raises
        - IllegalArgumentException: if `size` is negative
        """
        ...


    @staticmethod
    def newDataOutput(byteArrayOutputStream: "ByteArrayOutputStream") -> "ByteArrayDataOutput":
        """
        Returns a new ByteArrayDataOutput instance which writes to the given `ByteArrayOutputStream`. The given output stream is not reset before being written to by the
        returned `ByteArrayDataOutput` and new data will be appended to any existing content.
        
        Note that if the given output stream was not empty or is modified after the `ByteArrayDataOutput` is created, the contract for ByteArrayDataOutput.toByteArray will
        not be honored (the bytes returned in the byte array may not be exactly what was written via
        calls to `ByteArrayDataOutput`).

        Since
        - 17.0
        """
        ...


    @staticmethod
    def nullOutputStream() -> "OutputStream":
        """
        Returns an OutputStream that simply discards written bytes.

        Since
        - 14.0 (since 1.0 as com.google.common.io.NullOutputStream)
        """
        ...


    @staticmethod
    def limit(in: "InputStream", limit: int) -> "InputStream":
        """
        Wraps a InputStream, limiting the number of bytes which can be read.

        Arguments
        - in: the input stream to be wrapped
        - limit: the maximum number of bytes to be read

        Returns
        - a length-limited InputStream

        Since
        - 14.0 (since 1.0 as com.google.common.io.LimitInputStream)
        """
        ...


    @staticmethod
    def readFully(in: "InputStream", b: list[int]) -> None:
        """
        Attempts to read enough bytes from the stream to fill the given byte array, with the same
        behavior as DataInput.readFully(byte[]). Does not close the stream.

        Arguments
        - in: the input stream to read from.
        - b: the buffer into which the data is read.

        Raises
        - EOFException: if this stream reaches the end before reading all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    @staticmethod
    def readFully(in: "InputStream", b: list[int], off: int, len: int) -> None:
        """
        Attempts to read `len` bytes from the stream into the given array starting at `off`, with the same behavior as DataInput.readFully(byte[], int, int). Does not close
        the stream.

        Arguments
        - in: the input stream to read from.
        - b: the buffer into which the data is read.
        - off: an int specifying the offset into the data.
        - len: an int specifying the number of bytes to read.

        Raises
        - EOFException: if this stream reaches the end before reading all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    @staticmethod
    def skipFully(in: "InputStream", n: int) -> None:
        """
        Discards `n` bytes of data from the input stream. This method will block until the full
        amount has been skipped. Does not close the stream.

        Arguments
        - in: the input stream to read from
        - n: the number of bytes to skip

        Raises
        - EOFException: if this stream reaches the end before skipping all the bytes
        - IOException: if an I/O error occurs, or the stream does not support skipping
        """
        ...


    @staticmethod
    def readBytes(input: "InputStream", processor: "ByteProcessor"["T"]) -> "T":
        """
        Process the bytes of the given input stream using the given processor.

        Arguments
        - input: the input stream to process
        - processor: the object to which to pass the bytes of the stream

        Returns
        - the result of the byte processor

        Raises
        - IOException: if an I/O error occurs

        Since
        - 14.0
        """
        ...


    @staticmethod
    def read(in: "InputStream", b: list[int], off: int, len: int) -> int:
        """
        Reads some bytes from an input stream and stores them into the buffer array `b`. This
        method blocks until `len` bytes of input data have been read into the array, or end of
        file is detected. The number of bytes read is returned, possibly zero. Does not close the
        stream.
        
        A caller can detect EOF if the number of bytes read is less than `len`. All subsequent
        calls on the same stream will return zero.
        
        If `b` is null, a `NullPointerException` is thrown. If `off` is negative,
        or `len` is negative, or `off+len` is greater than the length of the array `b`, then an `IndexOutOfBoundsException` is thrown. If `len` is zero, then no bytes
        are read. Otherwise, the first byte read is stored into element `b[off]`, the next one
        into `b[off+1]`, and so on. The number of bytes read is, at most, equal to `len`.

        Arguments
        - in: the input stream to read from
        - b: the buffer into which the data is read
        - off: an int specifying the offset into the data
        - len: an int specifying the number of bytes to read

        Returns
        - the number of bytes read

        Raises
        - IOException: if an I/O error occurs
        - IndexOutOfBoundsException: if `off` is negative, if `len` is negative, or if
            `off + len` is greater than `b.length`
        """
        ...



"""
Python module generated from Java source file org.bukkit.boss.BarFlag

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.boss import *
from typing import Any, Callable, Iterable, Tuple


class BarFlag(Enum):

    DARKEN_SKY = 0
    """
    Darkens the sky like during fighting a wither.
    """
    PLAY_BOSS_MUSIC = 1
    """
    Tells the client to play the Ender Dragon boss music.
    """
    CREATE_FOG = 2
    """
    Creates fog around the world.
    """



"""
Python module generated from Java source file com.google.common.util.concurrent.Callables

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Supplier
from com.google.common.util.concurrent import *
from java.util.concurrent import Callable
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Callables:
    """
    Static utility methods pertaining to the Callable interface.

    Author(s)
    - Isaac Shum

    Since
    - 1.0
    """

    @staticmethod
    def returning(value: "T") -> "Callable"["T"]:
        """
        Creates a `Callable` which immediately returns a preset value each time it is called.
        """
        ...


    @staticmethod
    def asAsyncCallable(callable: "Callable"["T"], listeningExecutorService: "ListeningExecutorService") -> "AsyncCallable"["T"]:
        """
        Creates an AsyncCallable from a Callable.
        
        The AsyncCallable returns the ListenableFuture resulting from ListeningExecutorService.submit(Callable).

        Since
        - 20.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.configuration.Configuration

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.configuration import *
from typing import Any, Callable, Iterable, Tuple


class Configuration(ConfigurationSection):
    """
    Represents a source of configurable options and settings
    """

    def addDefault(self, path: str, value: "Object") -> None:
        """
        Sets the default value of the given path as provided.
        
        If no source Configuration was provided as a default
        collection, then a new MemoryConfiguration will be created to
        hold the new default value.
        
        If value is null, the value will be removed from the default
        Configuration source.

        Arguments
        - path: Path of the value to set.
        - value: Value to set the default to.

        Raises
        - IllegalArgumentException: Thrown if path is null.
        """
        ...


    def addDefaults(self, defaults: dict[str, "Object"]) -> None:
        """
        Sets the default values of the given paths as provided.
        
        If no source Configuration was provided as a default
        collection, then a new MemoryConfiguration will be created to
        hold the new default values.

        Arguments
        - defaults: A map of Path->Values to add to defaults.

        Raises
        - IllegalArgumentException: Thrown if defaults is null.
        """
        ...


    def addDefaults(self, defaults: "Configuration") -> None:
        """
        Sets the default values of the given paths as provided.
        
        If no source Configuration was provided as a default
        collection, then a new MemoryConfiguration will be created to
        hold the new default value.
        
        This method will not hold a reference to the specified Configuration,
        nor will it automatically update if that Configuration ever changes. If
        you require this, you should set the default source with .setDefaults(org.bukkit.configuration.Configuration).

        Arguments
        - defaults: A configuration holding a list of defaults to copy.

        Raises
        - IllegalArgumentException: Thrown if defaults is null or this.
        """
        ...


    def setDefaults(self, defaults: "Configuration") -> None:
        """
        Sets the source of all default values for this Configuration.
        
        If a previous source was set, or previous default values were defined,
        then they will not be copied to the new source.

        Arguments
        - defaults: New source of default values for this configuration.

        Raises
        - IllegalArgumentException: Thrown if defaults is null or this.
        """
        ...


    def getDefaults(self) -> "Configuration":
        """
        Gets the source Configuration for this configuration.
        
        If no configuration source was set, but default values were added, then
        a MemoryConfiguration will be returned. If no source was set
        and no defaults were set, then this method will return null.

        Returns
        - Configuration source for default values, or null if none exist.
        """
        ...


    def options(self) -> "ConfigurationOptions":
        """
        Gets the ConfigurationOptions for this Configuration.
        
        All setters through this method are chainable.

        Returns
        - Options for this configuration
        """
        ...



"""
Python module generated from Java source file java.io.Externalizable

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.io import ObjectInput
from java.io import ObjectOutput
from typing import Any, Callable, Iterable, Tuple


class Externalizable(Serializable):
    """
    Only the identity of the class of an Externalizable instance is
    written in the serialization stream and it is the responsibility
    of the class to save and restore the contents of its instances.
    
    The writeExternal and readExternal methods of the Externalizable
    interface are implemented by a class to give the class complete
    control over the format and contents of the stream for an object
    and its supertypes. These methods must explicitly
    coordinate with the supertype to save its state. These methods supersede
    customized implementations of writeObject and readObject methods.
    
    Object Serialization uses the Serializable and Externalizable
    interfaces.  Object persistence mechanisms can use them as well.  Each
    object to be stored is tested for the Externalizable interface. If
    the object supports Externalizable, the writeExternal method is called. If the
    object does not support Externalizable and does implement
    Serializable, the object is saved using
    ObjectOutputStream.  When an Externalizable object is
    reconstructed, an instance is created using the public no-arg
    constructor, then the readExternal method called.  Serializable
    objects are restored by reading them from an ObjectInputStream.
    
    An Externalizable instance can designate a substitution object via
    the writeReplace and readResolve methods documented in the Serializable
    interface.

    See
    - java.io.Serializable

    Since
    - 1.1
    """

    def writeExternal(self, out: "ObjectOutput") -> None:
        """
        The object implements the writeExternal method to save its contents
        by calling the methods of DataOutput for its primitive values or
        calling the writeObject method of ObjectOutput for objects, strings,
        and arrays.

        Arguments
        - out: the stream to write the object to

        Raises
        - IOException: Includes any I/O exceptions that may occur

        Serial Data
        - Overriding methods should use this tag to describe
                    the data layout of this Externalizable object.
                    List the sequence of element types and, if possible,
                    relate the element to a public/protected field and/or
                    method of this Externalizable class.
        """
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        """
        The object implements the readExternal method to restore its
        contents by calling the methods of DataInput for primitive
        types and readObject for objects, strings and arrays.  The
        readExternal method must read the values in the same sequence
        and with the same types as were written by writeExternal.

        Arguments
        - in: the stream to read data from in order to restore the object

        Raises
        - IOException: if I/O errors occur
        - ClassNotFoundException: If the class for an object being
                   restored cannot be found.
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ForwardingListeningExecutorService

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from java.util.concurrent import Callable
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ForwardingListeningExecutorService(ForwardingExecutorService, ListeningExecutorService):
    """
    A listening executor service which forwards all its method calls to another listening executor
    service. Subclasses should override one or more methods to modify the behavior of the backing
    executor service as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    **`default` method warning:** This class does *not* forward calls to `default` methods. Instead, it inherits their default implementations. When those implementations
    invoke methods, they invoke methods on the `ForwardingListeningExecutorService`.

    Author(s)
    - Isaac Shum

    Since
    - 10.0
    """

    def submit(self, task: "Callable"["T"]) -> "ListenableFuture"["T"]:
        ...


    def submit(self, task: "Runnable") -> "ListenableFuture"[Any]:
        ...


    def submit(self, task: "Runnable", result: "T") -> "ListenableFuture"["T"]:
        ...
